#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DMS Group v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ListGroup(command.Lister):
    _description = _('List DMS Groups')
    columns = ('ID', 'name', 'produced_messages', 'consumed_messages',
               'available_messages')

    def get_parser(self, prog_name):
        parser = super(ListGroup, self).get_parser(prog_name)
        parser.add_argument(
            'queue',
            metavar='<queue>',
            help=_('ID of the queue')
        )
        parser.add_argument(
            '--include_deadletter',
            action='store_true',
            help=_('Indicates whether to list dead letter parameters '
                   'in the response message.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dms

        queue = client.find_queue(parsed_args.queue)

        data = client.groups(
            queue=queue.id,
            include_deadletter=parsed_args.include_deadletter)

        if parsed_args.include_deadletter:
            self.columns = self.columns + (
                'produced_deadletters',
                'available_deadletters'
            )

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns,
                 ) for s in data))
        return table


class DeleteGroup(command.Command):
    _description = _('Delete DMS Group')

    def get_parser(self, prog_name):
        parser = super(DeleteGroup, self).get_parser(prog_name)
        parser.add_argument(
            'queue',
            metavar='<queue>',
            help=_('ID of the queue')
        )
        parser.add_argument(
            'group',
            metavar='<group>',
            nargs='+',
            help=_('ID of the Group')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.group:
            client = self.app.client_manager.dms
            for group in parsed_args.group:
                client.delete_group(queue=parsed_args.queue, group=group)


class CreateGroup(command.ShowOne):
    _description = _('Create DMS Group')
    columns = ('ID', 'name')

    def get_parser(self, prog_name):
        parser = super(CreateGroup, self).get_parser(prog_name)
        parser.add_argument(
            'queue',
            metavar='<queue>',
            help=_('ID of the queue')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the cluster.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dms

        obj = client.create_group(queue=parsed_args.queue,
                                  group=parsed_args.name)

        data = utils.get_item_properties(obj, self.columns)

        return (self.columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.function_graph.v2 import function
from otcextensions.sdk.function_graph.v2 import log

from openstack import _log

from otcextensions.tests.functional.sdk.function_graph import TestFg

_logger = _log.setup_logging('openstack')


class TestFunctionLog(TestFg):

    def setUp(self):
        super(TestFunctionLog, self).setUp()
        self.function = self.client.create_function(**TestFg.function_attrs)
        assert isinstance(self.function, function.Function)

        self.log = self.client.enable_lts_log()
        assert isinstance(self.log, log.Log)
        self.addCleanup(
            self.client.delete_function,
            self.function
        )

    def test_get_lts_log_details(self):
        lg = self.client.get_lts_log_settings(
            self.function)
        self.assertIsNotNone(lg)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''VolumeBackup Policy v1 action implementations'''
import argparse
# import json
import logging

from cliff import columns

# from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class BackupPolicy(columns.FormattableColumn):

    def human_readable(self):
        if self._value is None:
            return None

        return self._value.to_dict()

    def machine_readable(self):
        if self._value is None:
            return None

        return self._value.to_dict()


_formatters = {
    'scheduled_policy': BackupPolicy
}


class ListVolumeBackupPolicy(command.Lister):
    _description = _('List VolumeBackup Policies')
    columns = ('id', 'name', 'policy_resource_count', 'scheduled_policy',
               'tags')

    def get_parser(self, prog_name):
        parser = super(ListVolumeBackupPolicy, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):

        args = {}

        client = self.app.client_manager.volume_backup

        data = client.backup_policies(**args)

        return (
            self.columns,
            (utils.get_item_properties(
                s, self.columns, formatters=_formatters
            ) for s in data))


class DeleteVolumeBackupPolicy(command.Command):
    _description = _('Delete VolumeBackup Policy')

    def get_parser(self, prog_name):
        parser = super(DeleteVolumeBackupPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('id of the policy.')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.policy:
            client = self.app.client_manager.volume_backup
            client.delete_backup_policy(
                backup_policy=parsed_args.policy,
                ignore_missing=False)


class ShowVolumeBackupPolicy(command.ShowOne):
    _description = _('Show VolumeBackup Policy')
    columns = ('id', 'name', 'policy_resource_count', 'scheduled_policy',
               'tags')

    def get_parser(self, prog_name):
        parser = super(ShowVolumeBackupPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('ID or Name of the policy.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.volume_backup

        data = client.find_backup_policy(name_or_id=parsed_args.policy)

        return (
            self.columns,
            utils.get_item_properties(
                data, self.columns, formatters=_formatters
            ))


class CreateVolumeBackupPolicy(command.ShowOne):
    _description = _('Create VolumeBackup Policy')
    columns = ('id', 'name', 'policy_resource_count', 'scheduled_policy',
               'tags')

    def get_parser(self, prog_name):
        parser = super(CreateVolumeBackupPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the policy.')
        )
        parser.add_argument(
            '--start_time',
            metavar='<start_time>',
            required=True,
            help=_('Specifies the start time of the backup job. '
                   'You need to convert the local time to the '
                   'Coordinated Universal Time (UTC), and set the '
                   'start time to an integral hour point.\n'
                   'You can set multiple time points (at integral '
                   'hours only), and use commas (,) to separate one time '
                   'point from another. The value is in the HH:mm format.')
        )
        parser.add_argument(
            '--enable',
            action='store_true',
            help=_('Specifies whether the backup policy should be enabled '
                   '`status=ON`. By default `status=OFF` is used')
        )
        parser.add_argument(
            '--frequency',
            metavar='<frequency>',
            type=int,
            choices=range(1, 15),
            required=True,
            help=_('Specifies the backup interval (1 to 14 days). '
                   'Select either this parameter or week_frequency. If '
                   'you select both, this parameter is used.')
        )
        parser.add_argument(
            '--rentention_num',
            metavar='<rentention_num>',
            type=int,
            required=True,
            help=_('Specifies the retained number (minimum: 2) of backups. '
                   'Select either this parameter or rentention_day. '
                   'If you select both, this parameter is used.')
        )
        parser.add_argument(
            '--remain_first_backup_of_curMonth',
            action='store_true',
            help=_('Indicates whether to retain the first backup in the '
                   'current month. \n')
        )
        parser.add_argument(
            '--tag',
            metavar='<tag>',
            action='append',
            help=_('User defined tags to attach to the policy.\n'
                   'format: KEY=VALUE\n'
                   'KEY: [::ALPHANUM::]{0..36}\n'
                   'VALUE: [::ALPHANUM::]{0..43}\n'
                   '(Repeat multiple times for multiple tags)')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        attrs['name'] = parsed_args.name
        policy = {}
        policy['start_time'] = parsed_args.start_time
        if parsed_args.frequency:
            policy['frequency'] = parsed_args.frequency
        if parsed_args.rentention_num:
            policy['rentention_num'] = parsed_args.rentention_num
        if parsed_args.remain_first_backup_of_curMonth:
            policy['remain_first_backup_of_curMonth'] = 'Y'
        else:
            policy['remain_first_backup_of_curMonth'] = 'N'
        policy['status'] = 'ON' if parsed_args.enable else 'OFF'

        attrs['scheduled_policy'] = policy
        if parsed_args.tag:
            attrs['tags'] = []
            for tag in parsed_args.tag:
                tag_parts = tag.split('=')
                if 2 == len(tag_parts):
                    tag_dict = {
                        'key': tag_parts[0],
                        'value': tag_parts[1]
                    }
                    attrs['tags'].append(tag_dict)
                else:
                    msg = _('Cannot parse tag information')
                    raise argparse.ArgumentTypeError(msg)

        client = self.app.client_manager.volume_backup

        data = client.create_backup_policy(**attrs)

        return (
            self.columns,
            utils.get_item_properties(
                data, self.columns, formatters=_formatters
            ))


class UpdateVolumeBackupPolicy(command.ShowOne):
    _description = _('Update VolumeBackup Policy')
    columns = ('id', 'name', 'policy_resource_count', 'scheduled_policy',
               'tags')

    def get_parser(self, prog_name):
        parser = super(UpdateVolumeBackupPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'id',
            metavar='<id>',
            help=_('id of the policy.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the policy.')
        )
        parser.add_argument(
            '--start_time',
            metavar='<start_time>',
            help=_('Specifies the start time of the backup job. '
                   'You need to convert the local time to the '
                   'Coordinated Universal Time (UTC), and set the '
                   'start time to an integral hour point.\n'
                   'You can set multiple time points (at integral '
                   'hours only), and use commas (,) to separate one time '
                   'point from another. The value is in the HH:mm format.')
        )
        parser.add_argument(
            '--status',
            dest='status',
            type=sdk_utils.str2bool,
            nargs='?',
            help=_('Specifies whether or notthe backup policy should be '
                   'enabled (`true`, `false`)')
        )
        parser.add_argument(
            '--frequency',
            metavar='<frequency>',
            type=int,
            choices=range(1, 15),
            help=_('Specifies the backup interval (1 to 14 days). '
                   'Select either this parameter or week_frequency. If '
                   'you select both, this parameter is used.')
        )
        parser.add_argument(
            '--rentention_num',
            metavar='<rentention_num>',
            type=int,
            help=_('Specifies the retained number (minimum: 2) of backups. '
                   'Select either this parameter or rentention_day. '
                   'If you select both, this parameter is used.')
        )
        parser.add_argument(
            '--remain_first_backup_of_curMonth',
            dest='remain_first_backup_of_curMonth',
            type=sdk_utils.str2bool,
            nargs='?',
            help=_('Indicates whether or notto retain the first backup in the '
                   'current month (`true`, `false`). \n')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        policy = {}
        if parsed_args.start_time:
            policy['start_time'] = parsed_args.start_time
        if parsed_args.frequency:
            policy['frequency'] = parsed_args.frequency
        if parsed_args.rentention_num:
            policy['rentention_num'] = parsed_args.rentention_num
        if parsed_args.status is not None:
            policy['status'] = 'ON' if parsed_args.status else 'OFF'
        if parsed_args.remain_first_backup_of_curMonth is not None:
            if parsed_args.remain_first_backup_of_curMonth:
                policy['remain_first_backup_of_curMonth'] = 'Y'
            else:
                policy['remain_first_backup_of_curMonth'] = 'N'

        if policy:
            attrs['scheduled_policy'] = policy

        client = self.app.client_manager.volume_backup
        data = client.update_backup_policy(
            backup_policy=parsed_args.id,
            **attrs)

        return (
            self.columns,
            utils.get_item_properties(
                data, self.columns, formatters=_formatters
            ))


class ExecuteVolumeBackupPolicy(command.Command):
    _description = _('Execute VolumeBackup Policy')

    def get_parser(self, prog_name):
        parser = super(ExecuteVolumeBackupPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<id>',
            help=_('id of the policy.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.volume_backup
        client.execute_policy(backup_policy=parsed_args.policy)
        return


class LinkResourceToVolumeBackupPolicy(command.Command):
    _description = _('Link Resources to VolumeBackup Policy')

    def get_parser(self, prog_name):
        parser = super(LinkResourceToVolumeBackupPolicy, self).\
            get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<id>',
            help=_('id of the policy.')
        )
        parser.add_argument(
            '--volume',
            metavar='<volume_id>',
            action='append',
            help=_('id of volume to attach to the policy.'
                   '(repeat multiple times)')
        )
        return parser

    def take_action(self, parsed_args):
        volumes = parsed_args.volume

        client = self.app.client_manager.volume_backup
        client.link_resources_to_policy(
            backup_policy=parsed_args.policy,
            resources=volumes)
        return


class UnlinkResourceFromVolumeBackupPolicy(command.Command):
    _description = _('Unlink Resources to VolumeBackup Policy')

    def get_parser(self, prog_name):
        parser = super(UnlinkResourceFromVolumeBackupPolicy, self).\
            get_parser(prog_name)
        parser.add_argument(
            'policy',
            metavar='<id>',
            help=_('id of the policy.')
        )
        parser.add_argument(
            '--volume',
            metavar='<volume_id>',
            action='append',
            help=_('id of volume to remove from the policy.'
                   '(repeat multiple times)')
        )
        return parser

    def take_action(self, parsed_args):
        volumes = parsed_args.volume

        client = self.app.client_manager.volume_backup
        client.unlink_resources_of_policy(
            backup_policy=parsed_args.policy,
            resources=volumes)
        return



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts dataset version v2 action implementations"""
import logging

from cliff import columns as cliff_columns
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class VersionStatus(cliff_columns.FormattableColumn):
    CHOICES_MAP = {
        0: "creating",
        1: "running",
        2: "deleting",
        3: "deleted",
        4: "error",
    }
    STR = "\n".join(f"{key}: {value}" for key, value in CHOICES_MAP.items())

    def human_readable(self):
        return self.CHOICES_MAP.get(self._value, str(self._value))


class ListDatasetVersions(command.Lister):
    _description = _("This API is used to query the Dataset Version list.")
    columns = (
        "Version Id",
        "Version Name",
        "Version Format",
        "Status",
        "Created At",
    )

    def get_parser(self, prog_name):
        parser = super(ListDatasetVersions, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        parser.add_argument(
            "--status",
            metavar="<status>",
            choices=[0, 1, 2, 3, 4],
            type=int,
            help=_(
                "Status of a dataset version. The options are as follows:"
                "\n0: creating"
                "\n1: running"
                "\n2: deleting"
                "\n3: deleted"
                "\n4: error"
            ),
        )
        parser.add_argument(
            "--train-evaluate-ratio",
            metavar="<train_evaluate_ratio>",
            help=_("Version split ratio for version filtering."),
        )
        parser.add_argument(
            "--version-format",
            metavar="<version_format>",
            choices=[0, 1, 2],
            type=int,
            help=_(
                "Format of a dataset version. The options are as follows:"
                "\n0: default format"
                "\n1: CarbonData (supported only by table datasets)"
                "\n2: CSV"
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        query_params = {}
        for arg in ("status", "train_evaluate_ratio", "version_format"):
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                query_params[arg] = val

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        data = client.dataset_versions(dataset.id, **query_params)

        formatters = {
            "Status": VersionStatus,
            "Created At": cli_utils.UnixTimestampFormatter,
        }
        return (
            self.columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )


class CreateDatasetVersion(command.ShowOne):
    _description = _("Create a ModelArts dataset version")

    def get_parser(self, prog_name):
        parser = super(CreateDatasetVersion, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        parser.add_argument(
            "--name",
            metavar="<name>",
            dest="version_name",
            help=_(
                "version_name. The value is a string of 1 to 32 "
                "characters consisting of only digits, letters, "
                "underscores (_), and hyphens (-). "
                "Example value: dataset"
            ),
        )
        parser.add_argument(
            "--description",
            metavar="<description>",
            help=_(
                "Dataset description. The value is a string of 0 to 256 "
                "characters. By default, this parameter is left blank."
            ),
        )
        parser.add_argument(
            "--version-format",
            metavar="<version_format>",
            help=_(
                "Format of the exported version file, which is "
                "case insensitive."
            ),
        )
        parser.add_argument(
            "--remove-sample-usage",
            metavar="<remove_sample_usage>",
            help=_(
                "Whether to clear the usage information of dataset "
                "samples. The default value is true."
            ),
        )
        parser.add_argument(
            "--export-images",
            metavar="<export_images>",
            help=_(
                "Whether to export images to the version output directory "
                "during publishing. The default value is false."
            ),
        )
        parser.add_argument(
            "--train-evaluate-sample-ratio",
            metavar="<train_evaluate_sample_ratio>",
            help=_(
                "Ratio that splits the labeled data into training and "
                "validation sets during publishing. The value must be "
                "a decimal between 0 and 1. "
            ),
        )
        parser.add_argument(
            "--clear_hard_property",
            metavar="<clear_hard_property>",
            help=_(
                "Whether to clear hard example properties. "
                "The default value is true."
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        attrs = {}
        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        if parsed_args.version_name:
            attrs["version_name"] = parsed_args.version_name
        if parsed_args.version_format:
            attrs["version_format"] = parsed_args.version_format
        if parsed_args.remove_sample_usage:
            attrs["remove_sample_usage"] = parsed_args.remove_sample_usage
        if parsed_args.export_images:
            attrs["export_images"] = parsed_args.export_images
        if parsed_args.train_evaluate_sample_ratio:
            attrs["train_evaluate_sample_ratio"] = (
                parsed_args.train_evaluate_sample_ratio
            )
        if parsed_args.clear_hard_property:
            attrs["clear_hard_property"] = parsed_args.clear_hard_property
        if parsed_args.description:
            attrs["description"] = parsed_args.description

        obj = client.create_dataset_version(dataset.id, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return display_columns, data


class ShowDatasetVersion(command.ShowOne):
    _description = _("Show details of a MA dataset version")

    def get_parser(self, prog_name):
        parser = super(ShowDatasetVersion, self).get_parser(prog_name)

        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )

        parser.add_argument(
            "versionId",
            metavar="<versionId>",
            help=_("Dataset version id."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2

        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )
        obj = client.get_dataset_version(dataset.id, parsed_args.versionId)

        formatters = {
            "status": VersionStatus,
            "created_at": cli_utils.UnixTimestampFormatter,
            "label_stats": cli_utils.YamlFormat,
        }

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=formatters)

        return (display_columns, data)


class DeleteDatasetVersion(command.Command):
    _description = _("Delete ModelArts Dataset Version.")

    def get_parser(self, prog_name):
        parser = super(DeleteDatasetVersion, self).get_parser(prog_name)
        parser.add_argument(
            "dataset",
            metavar="<dataset>",
            help=_("Dataset Id or name."),
        )
        parser.add_argument(
            "versionId",
            metavar="<versionId>",
            help=_("Version Id of the dataset."),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv2
        dataset = client.find_dataset(
            parsed_args.dataset, ignore_missing=False
        )

        client.delete_dataset_version(dataset.id, parsed_args.versionId)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from openstackclient.tests.unit import utils
from osc_lib import utils as _osc_lib_utils

from otcextensions.sdk.modelartsv1.v1 import builtin_model
from otcextensions.sdk.modelartsv1.v1 import devenv
from otcextensions.sdk.modelartsv1.v1 import job_engine
from otcextensions.sdk.modelartsv1.v1 import job_flavor
from otcextensions.sdk.modelartsv1.v1 import model
from otcextensions.sdk.modelartsv1.v1 import service
from otcextensions.sdk.modelartsv1.v1 import service_cluster
from otcextensions.sdk.modelartsv1.v1 import service_event
from otcextensions.sdk.modelartsv1.v1 import service_flavor
from otcextensions.sdk.modelartsv1.v1 import service_log
from otcextensions.sdk.modelartsv1.v1 import service_monitor
from otcextensions.sdk.modelartsv1.v1 import training_job
from otcextensions.sdk.modelartsv1.v1 import training_job_config
from otcextensions.sdk.modelartsv1.v1 import training_job_version
from otcextensions.sdk.modelartsv1.v1 import visualization_job
from otcextensions.tests.unit.osclient import test_base
from otcextensions.tests.unit.sdk.modelartsv1.v1 import examples


def gen_data(obj, columns, formatters=None):
    """Fill expected data tuple based on columns list"""
    return _osc_lib_utils.get_item_properties(
        obj, columns, formatters=formatters
    )


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list"""
    return tuple(data.get(attr, "") for attr in columns)


class TestModelartsv1(utils.TestCommand):
    def setUp(self):
        super(TestModelartsv1, self).setUp()

        self.app.client_manager.modelartsv1 = mock.Mock()

        self.client = self.app.client_manager.modelartsv1


class FakeBuiltInModel(test_base.Fake):
    """Fake one or more Modelarts built-in model."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts built-in model.

        :return:
            A FakeResource object, with id, name and so on
        """
        return builtin_model.BuiltInModel(**examples.BUILT_IN_MODEL)


class FakeDevenv(test_base.Fake):
    """Fake one or more Modelarts devenv."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts devenv.

        :return:
            A FakeResource object, with id, name and so on
        """
        return devenv.Devenv(**examples.DEVENV)


class FakeModel(test_base.Fake):
    """Fake one or more Modelarts model."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts model.

        :return:
            A FakeResource object, with id, name and so on
        """
        return model.Model(**examples.MODEL)


class FakeJobFlavor(test_base.Fake):
    """Fake one or more Modelarts model."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts model.

        :return:
            A FakeResource object, with id, name and so on
        """

        return job_flavor.JobFlavor(**examples.JOB_FLAVOR)


class FakeJobEngine(test_base.Fake):
    """Fake one or more Modelarts model."""

    @classmethod
    def generate(cls):
        """Create a fake job resource engine.

        :return:
            A FakeResource object, with id, name and so on
        """
        object_info = {
            "engine_type": 4,
            "engine_name": "Caffe",
            "engine_id": 8,
            "engine_version": "Caffe-1.0.0-python2.7",
        }

        return job_engine.JobEngine(**object_info)


class FakeService(test_base.Fake):
    """Fake one or more Modelarts service."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts service.

        :return:
            A FakeResource object, with id, name and so on
        """
        return service.Service(**examples.SERVICE)


class FakeServiceLog(test_base.Fake):
    """Fake one or more Modelarts service log."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts service Log.

        :return:
            A FakeResource object, with id, name and so on
        """
        return service_log.ServiceLog(**examples.SERVICE_LOG)


class FakeServiceEvent(test_base.Fake):
    """Fake one or more Modelarts service event."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts service Event.

        :return:
            A FakeResource object.
        """
        object_info = {
            "occur_time": 1562597251764,
            "event_type": "normal",
            "event_info": "start to deploy service",
        }
        return service_event.ServiceEvent(**object_info)


class FakeServiceSpecification(test_base.Fake):
    """Fake one or more Modelarts service specification."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts service specification.

        :return:
            A FakeResource object.
        """
        object_info = {
            "specification": "modelarts.vm.gpu.v100",
            "billing_spec": "modelarts.vm.gpu.v100",
            "is_open": True,
            "spec_status": "normal",
            "is_free": False,
            "over_quota": False,
            "extend_params": 1,
        }
        return service_flavor.ServiceFlavor(**object_info)


class FakeServiceMonitor(test_base.Fake):
    """Fake one or more Modelarts service monitor."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts service monitor.

        :return:
            A FakeResource object.
        """
        monitor = service_monitor.ServiceMonitor(**examples.SERVICE_MONITOR)
        monitor.cpu_core = f"{monitor.cpu_core_usage}/{monitor.cpu_core_total}"
        monitor.cpu_memory = (
            f"{monitor.cpu_memory_usage}/{monitor.cpu_memory_total}"
        )
        monitor.gpu = f"{monitor.gpu_usage}/{monitor.gpu_total}"
        return monitor


class FakeServiceCluster(test_base.Fake):
    """Fake one or more Modelarts service cluster."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts service cluster.

        :return:
            A FakeResource object.
        """
        cluster = service_cluster.ServiceCluster(**examples.SERVICE_CLUSTER)
        cluster.allocatable_resources = {
            "cpu_cores": cluster.allocatable_cpu_cores,
            "memory": cluster.allocatable_memory,
            "gpus": cluster.allocatable_gpus,
        }
        return cluster


class FakeTrainingJob(test_base.Fake):
    """Fake one or more Modelarts visualization job."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts visualization job.

        :return:
            A FakeResource object, with id, name and so on
        """
        return training_job.TrainingJob(**examples.TRAINING_JOB)


class FakeTrainingJobVersion(test_base.Fake):
    """Fake one or more Modelarts training job version."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts training job version.

        :return:
            A FakeResource object, with id, name and so on
        """
        return training_job_version.TrainingJobVersion(
            **examples.TRAINING_JOB_VERSION
        )


class FakeTrainingJobConfig(test_base.Fake):
    """Fake one or more Modelarts training job configuration."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts training job configuration.

        :return:
            A FakeResource object, with id, name and so on
        """
        return training_job_config.TrainingJobConfig(
            **examples.TRAINING_JOB_CONFIG
        )


class FakeVisualizationJob(test_base.Fake):
    """Fake one or more Modelarts visualization job."""

    @classmethod
    def generate(cls):
        """Create a fake Modelarts visualization job.

        :return:
            A FakeResource object, with id, name and so on
        """
        return visualization_job.VisualizationJob(**examples.VISUALIZATION_JOB)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.imsv2.v2 import image

EXAMPLE = {
    "name": "ims_test_file",
    "description": "Create an image from a file in an OBS bucket",
    "image_url": "imsv2-image:centos70.qcow2",
    "os_version": "CentOS 7.0 64bit",
    "min_disk": 40,
    "image_tags": [{"key": "key2", "value": "value2"},
                   {"key": "key1", "value": "value1"}]
}


class TestImage(base.TestCase):

    def test_basic(self):
        sot = image.Image()
        path = '/cloudimages'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = image.Image(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['image_url'], sot.image_url)
        self.assertEqual(EXAMPLE['os_version'], sot.os_version)
        self.assertEqual(EXAMPLE['min_disk'], sot.min_disk)
        self.assertEqual(EXAMPLE['image_tags'], sot.image_tags)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid

import mock
from openstackclient.tests.unit import utils
from osc_lib import utils as _osc_lib_utils

from otcextensions.sdk.dws.v1 import cluster
from otcextensions.sdk.dws.v1 import flavor
from otcextensions.sdk.dws.v1 import snapshot
from otcextensions.tests.unit.osclient import test_base

# from collections import defaultdict
# from datetime import datetime


def gen_data(obj, columns, formatters=None):
    """Fill expected data tuple based on columns list"""
    return _osc_lib_utils.get_item_properties(
        obj, columns, formatters=formatters
    )


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list"""
    return tuple(data.get(attr, '') for attr in columns)


class TestDws(utils.TestCommand):
    def setUp(self):
        super(TestDws, self).setUp()

        self.app.client_manager.dws = mock.Mock()

        self.client = self.app.client_manager.dws


class FakeCluster(test_base.Fake):
    """Fake one or more DWS Clusters."""

    @classmethod
    def generate(cls):
        """Create a fake DWS Cluster.

        :return:
            A FakeResource object, with id, name and so on
        """

        object_info = {
            'action_progress': {},
            'availability_zone': 'eu-de-01',
            'created': '2023-01-16T02:11:19',
            # "elb": None,
            'endpoints': [{
                'connect_info': 'test.example.com',
                'jdbc_url': 'jdbc:postgresql://test.example.com:8000/gaussdb',
            }],
            'enterprise_project_id': uuid.uuid4().hex,
            'guest_agent_version': '8.1.1.202',
            'id': uuid.uuid4().hex,
            'logical_cluster_initialed': False,
            'logical_cluster_mode': False,
            'maintain_window': {
                'day': 'Thu',
                'end_time': '02:00',
                'start_time': '22:00',
            },
            'name': 'test-dws-6433a577',
            'node_type': 'dws.m3.xlarge',
            'node_type_id': uuid.uuid4().hex,
            'nodes': [{'id': uuid.uuid4().hex, 'status': '200'}],
            'number_of_free_node': 0,
            'number_of_node': 6,
            'parameter_group': {
                'id': uuid.uuid4().hex,
                'name': 'parameterGroupFor_' + uuid.uuid4().hex,
                'status': 'In-Sync',
            },
            'plugins': [
                {
                    'aarchType': 'all',
                    'autoInstall': 'install|upgrade',
                    'datastoreType': 'dws',
                    'datastoreVersion': '8.1.1.202',
                    'id': uuid.uuid4().hex,
                    'packageName': '8.1.1.202-inspect-xxxx.tar.gz',
                    'status': '0',
                    'type': 'inspect',
                    'updateTime': 1666792707000,
                    'version': '8.1.1.202',
                }
            ],
            'port': 8000,
            'private_ip': ['192.168.1.192', '192.168.1.15', '192.168.1.73'],
            'public_endpoints': [{
                'jdbc_url': 'jdbc:postgresql://test.example.com:8000/gaussdb',
                'public_connect_info': 'test.example.com',
            }],
            'public_ip': {
                'eip_address': '1.2.3.4',
                'eip_id': uuid.uuid4().hex,
                'public_bind_type': 'auto_assign',
            },
            'recent_event': 9,
            'security_group_id': uuid.uuid4().hex,
            'spec_version': 'v1.0',
            'status': 'AVAILABLE',
            'sub_status': 'NORMAL',
            'subnet_id': uuid.uuid4().hex,
            'tags': [{'key': 'key2', 'value': 'value2'}],
            'task_status': 'SNAPSHOTTING',
            'updated': '2023-01-16T02:11:19',
            'use_logical_cluster': False,
            'user_name': 'dbadmin',
            'version': '8.1.1.202',
            'vpc_id': uuid.uuid4().hex,
        }
        return cluster.Cluster(**object_info)


class FakeSnapshot(test_base.Fake):
    """Fake one or more DWS Snapshot"""

    @classmethod
    def generate(cls):
        """Create a fake DWS Snapshot.

        :return:
            A FakeResource object, with id, name and so on
        """
        object_info = {
            'id': uuid.uuid4().hex,
            'name': 'snapshot-1',
            'description': 'snapshot description',
            'started': '2016-08-23T03:59:23Z',
            'finished': '2016-08-23T04:01:40Z',
            'size': 500,
            'status': 'AVAILABLE',
            'type': 'MANUAL',
            'cluster_id': uuid.uuid4().hex,
        }
        return snapshot.Snapshot(**object_info)


class FakeFlavor(test_base.Fake):
    """Fake one or more Flavor"""

    @classmethod
    def generate(cls):
        """Create a fake DWS Flavor.

        :return:
            A FakeResource object, with id, name and so on
        """
        object_info = {
            'id': uuid.uuid4().hex,
            'availabileZones': 'eu-de-02,eu-de-01',
            'detail': [
                {'value': '4', 'type': 'vCPU'},
                {'value': '160', 'type': 'SSD', 'unit': 'GB'},
                {'value': '32', 'type': 'mem', 'unit': 'GB'},
                {'value': 'eu-de-02,eu-de-01', 'type': 'availableZones'},
            ],
            'disk_size': 160,
            'disk_type': 'SSD',
            'spec_name': 'dws.m3.xlarge',
            'mem': 32,
            'vCPU': 4,
        }

        return flavor.Flavor(**object_info)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack
# from openstack import exceptions
# from openstack import utils

from otcextensions.tests.functional import base
import time

_logger = openstack._log.setup_logging('openstack')


class TestMessage(base.BaseFunctionalTest):
    QUEUE_ALIAS = 'sdk_test_queue'
    queues = []
    groups = []
    messages = []
    received_messages = []

    def setUp(self):
        super(TestMessage, self).setUp()
        try:
            self.queue = self.conn.dms.create_queue(
                name=TestMessage.QUEUE_ALIAS
            )

        except openstack.exceptions.BadRequestException:
            self.queue = self.conn.dms.find_queue(TestMessage.QUEUE_ALIAS)

        self.queues.append(self.queue)

        try:
            self.group = self.conn.dms.create_group(
                self.queue, "test_group"
            )

        except openstack.exceptions.DuplicateResource:
            self.queue = self.conn.dms.groups(self.queue)

        self.groups.append(self.group)

    def tearDown(self):
        try:
            for queue in self.queues:
                if queue.id:
                    self.conn.dms.delete_queue(queue)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)
        super(TestMessage, self).tearDown()

    def test_list(self):
        self.queues = list(self.conn.dms.queues())
        self.assertGreaterEqual(len(self.queues), 0)
        if len(self.queues) > 0:
            queue = self.queues[0]
            q = self.conn.dms.get_queue(queue=queue.id)
            self.assertIsNotNone(q)

    def test_group(self):
        self.queues = list(self.conn.dms.queues())
        # self.assertGreaterEqual(len(self.queues), 0)
        if len(self.queues) > 0:
            # queue = self.queues[0]
            # q = self.conn.dms.get_queue(queue=queue.id)
            # self.assertIsNotNone(q)
            try:
                self.group = self.conn.dms.create_group(
                    self.queue, "test_group2"
                )

            except openstack.exceptions.BadRequestException:
                self.queue = self.conn.dms.groups(self.queue)

            self.groups.append(self.group)

    # OS_TEST_TIMEOUT=60 is needed due to testbed slowness
    def test_message(self):
        self.queues = list(self.conn.dms.queues())
        # self.assertGreaterEqual(len(self.queues), 0)
        if len(self.queues) > 0:
            # queue = self.queues[0]
            # q = self.conn.dms.get_queue(queue=queue.id)
            time.sleep(50)

            # self.assertIsNotNone(q)
            self.message = self.conn.dms.send_messages(
                self.queue,
                messages=[
                    {"body": "TEST11",
                        "attributes":
                            {
                                "attribute1": "value1",
                                "attribute2": "value2"}}
                ]
            )
            #        ,{ "body" : { "foo" : "test02" },
            #        "attributes" : {
            #            "attribute1" : "value1",
            #            "attribute2" : "value2" } }

            self.messages.append(self.message)
            try:
                self.group = self.conn.dms.create_group(
                    self.queue, "test_group3"
                )

            except openstack.exceptions.BadRequestException:
                self.queue = self.conn.dms.groups(self.queue)

            self.groups.append(self.group)

            self.received_messages = self.conn.dms.consume_message(
                self.queue,
                self.group
            )
            self.assertGreaterEqual(len(list(self.received_messages)), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import uuid

import mock
from openstackclient.tests.unit import utils
from osc_lib import utils as _osc_lib_utils

from otcextensions.sdk.css.v1 import cluster
from otcextensions.sdk.css.v1 import cluster_upgrade_status
from otcextensions.sdk.css.v1 import cluster_image
from otcextensions.sdk.css.v1 import flavor
from otcextensions.sdk.css.v1 import snapshot
from otcextensions.tests.unit.osclient import test_base

# from datetime import datetime


def gen_data(obj, columns, formatters=None):
    """Fill expected data tuple based on columns list"""
    return _osc_lib_utils.get_item_properties(
        obj, columns, formatters=formatters
    )


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list"""
    return tuple(data.get(attr, '') for attr in columns)


class TestCss(utils.TestCommand):
    def setUp(self):
        super(TestCss, self).setUp()

        self.app.client_manager.css = mock.Mock()

        self.client = self.app.client_manager.css


class FakeCluster(test_base.Fake):
    """Fake one or more Nat Gateways."""

    @classmethod
    def generate(cls):
        """Create a fake CSS Cluster.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            'actionProgress': {},
            'actions': [],
            'authorityEnable': False,
            'backupAvailable': False,
            'bandwidthSize': 0,
            'cmk_id': 'cmk-uuid',
            'created': '2024-04-02T23:23:31',
            'datastore': {'type': 'elasticsearch', 'version': '7.10.2'},
            'diskEncrypted': False,
            'elbWhiteList': {'whiteList': '', 'enableWhiteList': False},
            'endpoint': [
                '192.168.0.194:9200',
                '192.168.0.12:9200',
                '192.168.0.66:9200',
            ],
            'enterpriseProjectId': '0',
            'httpsEnable': False,
            'id': uuid.uuid4().hex,
            'instances': [
                {
                    'azCode': 'eu-de-01',
                    'id': 'node-id',
                    'ip': '192.168.0.194',
                    'name': 'test-cluster-ess-esn-1-1',
                    'specCode': 'css.xlarge.2',
                    'status': '200',
                    'type': 'ess',
                    'volume': {
                        'type': 'COMMON',
                        'size': 100,
                    },
                },
            ],
            'name': 'test-cluster',
            'period': False,
            'publicIp': '1.2.3.4',
            'publicKibanaResp': None,
            'securityGroupId': 'sg-id',
            'status': 'AVAILABLE',
            'status_code': '200',
            'subnetId': 'network-id',
            'tags': [
                {'key': 'key0', 'value': 'value0'},
                {'key': 'key1', 'value': 'value1'},
            ],
            'updated': '2024-04-03T10:37:44',
            'vpcId': 'router-id',
        }
        obj = cluster.Cluster(**object_info)
        setattr(obj, 'num_nodes', len(obj.nodes))
        return obj


class FakeClusterImage(test_base.Fake):
    @classmethod
    def generate(cls):
        object_info = {
            'imageInfoList': [
                {
                    'datastoreType': 'elasticsearch',
                    'datastoreVersion': '7.9.3',
                    'displayName': '7.9.3_23.3.1_0823',
                    'id': '9a07921e-0168-43e2-b5b6-2aa63912bdf3',
                    'imageDesc': 'The latest image of version 7.9.3 will ...',
                    'priority': 6,
                }
            ],
            'needUploadUpgradePlugin': False,
        }
        return cluster_image.ClusterImage(**object_info)


class FakeClusterUpgradeStatus(test_base.Fake):
    @classmethod
    def generate(cls):
        object_info = {
            'agencyName': 'css_upgrade_agency',
            'completedNodes': 'asomogyi_0506-ess-esn-1-1',
            'endTime': '2024-07-15T15:11:31',
            'executeTimes': '1',
            'id': '7ebd2c2f-0a4f-429c-b3b3-4cf263b93d5b',
            'imageInfo': {
                'datastoreType': 'opensearch',
                'datastoreVersion': '1.3.6',
                'displayName': '1.3.6_23.5.1_0123',
                'id': 'cff076f4-84c3-42eb-842e-b93a2ed4609a',
                'imageDesc': 'Provide services based on Opensearch...',
                'priority': 11,
            },
            'startTime': '2024-07-15T14:45:46',
            'status': 'SUCCESS',
            'totalNodes': 'asomogyi_0506-ess-esn-1-1',
        }

        return cluster_upgrade_status.ClusterUpgradeStatus(**object_info)


class FakeSnapshot(test_base.Fake):
    """Fakse one or more Snapshot"""

    @classmethod
    def generate(cls):
        """Create a fake CSS Snapshot.

        :return:
            A FakeResource object, with id, name and so on
        """
        object_info = {
            'created': '2018-03-07T07:34:47',
            'datastore': {'type': 'elasticsearch', 'version': '6.2.3'},
            'description': '',
            'id': 'id-' + uuid.uuid4().hex,
            'clusterId': 'cluster_id-' + uuid.uuid4().hex,
            'clusterName': 'Es-xfx',
            'name': 'snapshot-002',
            'status': 'COMPLETED',
            'updated': '2018-03-07T07:40:12',
            'backupType': '1',
            'backupMethod': 'manual',
            'backupExpectedStartTime': None,
            'backupKeepDay': None,
            'backupPeriod': None,
            'indices': '.kibana,website2',
            'totalShards': 6,
            'failedShards': 0,
            'version': '6.2.3',
            'restoreStatus': 'success',
            'startTime': 1520408087099,
            'endTime': 1520408412219,
            'bucketName': 'obs-b8ed',
        }
        return snapshot.Snapshot(**object_info)


class FakeSnapshotPolicy(test_base.Fake):
    """Fake one or more Snapshot Policy."""

    @classmethod
    def generate(cls):
        """Create a fake CSS Snapshot Policy.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            'keepday': 2,
            'period': '16:00 GMT+08:00',
            'prefix': 'snapshot',
            'bucket': 'test-bucket',
            'basePath': 'css_repository/tests',
            'agency': 'usearch',
            'enable': 'true',
            'snapshotCmkId': 'kms-' + uuid.uuid4().hex,
        }
        return snapshot.SnapshotPolicy(**object_info)


class FakeFlavor(test_base.Fake):
    """Fake one or more Flavors."""

    @classmethod
    def generate(cls):
        """Create a fake CSS Snapshot Policy.

        :return:
            A FakeResource object, with id, name and so on
        """
        # Set default attributes.
        object_info = {
            'type': 'ess',
            'version': '7.6.2',
            'cpu': 1,
            'ram': 8,
            'name': 'css.medium.8',
            'region': 'eu-de',
            'diskrange': '40,640',
            'availableAZ': 'eu-de-01,eu-de-02,eu-de-03',
            'flavor_id': uuid.uuid4().hex,
        }
        return flavor.Flavor(**object_info)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Config(resource.Resource):

    resources_key = 'redis_config'

    base_path = '/instances/%(instance_id)s/configs'

    # capabilities
    allow_list = True
    allow_commit = True
    commit_method = "PUT"

    # Properties
    #: Instance ID
    instance_id = resource.URI('instance_id')
    #: Parameter description
    description = resource.Body('description')
    #: Parameter default value
    default_value = resource.Body('default_value')
    #: Parameter ID
    id = resource.Body('param_id', alternate_id=True)
    #: Parameter Name
    name = resource.Body('param_name')
    #: Parameter value
    value = resource.Body('param_value')
    #: Parameter value range
    value_range = resource.Body('value_range')
    #: Parameter value type
    value_type = resource.Body('value_type')

    def _construct_dict_for_update(self, obj):
        return {
            key: obj[key] for key in ['param_id', 'param_value', 'param_name']
        }

    def _update(self, session, params):
        """Update parameters of the instance
        """
        filter_params = [
            self._construct_dict_for_update(obj) for obj in params
        ]
        body = {
            'redis_config': filter_params
        }
        uri = self.base_path % self._uri.attributes
        response = session.put(
            uri,
            json=body)
        return self._translate_response(response, False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# import six
from openstack import resource


class Checkpoint(resource.Resource):
    base_path = '/checkpoints'

    allow_create = True
    allow_delete = True
    allow_fetch = True

    # Properties
    #: Name of the app, which is the unique identifier of a user data "
    #:  consumption program.
    app_name = resource.Body('app_name')
    #: Type of the checkpoint
    #:  \nLAST_READ: Only sequence numbers are recorded in databases.
    checkpoint_type = resource.Body('checkpoint_type')
    #: Metadata information of the consumer application.
    #:  The metadata information can contain a maximum of 1,000 characters.
    metadata = resource.Body('metadata')
    #: Partition ID of the stream.
    partition_id = resource.Body('partition_id')
    #: Sequence number to be submitted, which is used to record the
    #:  consumption checkpoint of the stream.
    sequence_number = resource.Body('sequence_number')
    #: Name of the stream.
    stream_name = resource.Body('stream_name')

    def get_checkpoint(self, session, **params):
        """Querying Checkpoint Details.
        """
        response = session.get(self.base_path, params=params)
        self._translate_response(response)
        return self

    def delete_checkpoint(self, session, **params):
        """Deleting Checkpoint.
        """
        response = session.delete(self.base_path, params=params)
        self._translate_response(response, has_body=False)
        return self



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.auto_scaling.v1 import policy
from otcextensions.tests.unit.osclient.auto_scaling.v1 import fakes


class TestAutoScalingPolicy(fakes.TestAutoScaling):

    def setUp(self):
        super(TestAutoScalingPolicy, self).setUp()
        self.client = self.app.client_manager.auto_scaling


class TestListAutoScalingPolicy(TestAutoScalingPolicy):

    policies = fakes.FakePolicy.create_multiple(3)

    columns = ['ID', 'Name']

    data = []

    for s in policies:
        data.append((
            s.id,
            s.name,
            # s.scaling_group_id,
            # s.status,
            # s.type,
            # s.alarm_id,
            # s.scheduled_policy,
            # s.scaling_policy_action,
            # s.cool_down_time,
        ))

    def setUp(self):
        super(TestListAutoScalingPolicy, self).setUp()

        self.cmd = policy.ListAutoScalingPolicy(self.app, None)

        self.client.groups = mock.Mock()

    def test_list(self):
        arglist = [
            '--group', 'grp',
        ]

        verifylist = [
            ('group', 'grp')
        ]

        # Set the response
        self.client.groups.side_effect = [
            self.policies
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        grp_mock = mock.Mock()
        grp_mock.id = 2

        self.client.find_group = mock.Mock(return_value=grp_mock)
        # Set the response
        self.client.policies.side_effect = [
            self.policies
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.policies.assert_called_once_with(group=grp_mock.id)

        self.assertEqual(self.columns, list(columns))
        self.assertEqual(self.data, list(data))


class TestShowAutoScalingPolicy(TestAutoScalingPolicy):

    _policy = fakes.FakePolicy.create_one()

    columns = ['ID', 'Name', 'scaling_group_id', 'status',
               'type', 'alarm_id', 'scheduled_policy',
               'scaling_policy_action', 'cool_down_time'
               ]

    data = (
        _policy.id,
        _policy.name,
        _policy.scaling_group_id,
        _policy.status,
        _policy.type,
        _policy.alarm_id,
        _policy.scheduled_policy,
        _policy.scaling_policy_action,
        _policy.cool_down_time,
    )

    def setUp(self):
        super(TestShowAutoScalingPolicy, self).setUp()

        self.cmd = policy.ShowAutoScalingPolicy(self.app, None)

        self.client.find_group = mock.Mock()

    def test_show_default(self):
        arglist = [
            'test_policy'
        ]
        verifylist = [
            ('policy', 'test_policy')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.get_policy.side_effect = [
            self._policy
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_policy.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateAutoScalingPolicy(TestAutoScalingPolicy):

    columns = ['ID', 'Name', 'scaling_group_id', 'status',
               'type', 'alarm_id', 'scheduled_policy',
               'scaling_policy_action', 'cool_down_time'
               ]

    _obj = fakes.FakePolicy.create_one()

    data = (
        _obj.id,
        _obj.name,
        _obj.scaling_group_id,
        _obj.status,
        _obj.type,
        _obj.alarm_id,
        _obj.scheduled_policy,
        _obj.scaling_policy_action,
        _obj.cool_down_time,
    )

    def setUp(self):
        super(TestCreateAutoScalingPolicy, self).setUp()

        self.cmd = policy.CreateAutoScalingPolicy(self.app, None)

        self.client.create_policy = mock.Mock()

    def test_create(self):
        arglist = [
            '--group', 'group1',
            '--type', 'ALARM',
            '--cool-down-time', '1',
            '--alarm-id', 'alarm1',
            '--action-operation', 'ADD',
            '--action-instance-number', '7',
            '--launch-time', 'launch_time1',
            '--recurrence-type', 'recurrence_type1',
            '--recurrence-value', 'recurrence_value1',
            '--start-time', 'st1',
            '--end-time', 'et1',

            'test_name'
        ]
        verifylist = [
            ('group', 'group1'),
            ('cool_down_time', 1),
            ('type', 'ALARM'),
            ('alarm_id', 'alarm1'),
            ('action_operation', 'ADD'),
            ('action_instance_number', 7),
            ('launch_time', 'launch_time1'),
            ('recurrence_type', 'recurrence_type1'),
            ('recurrence_value', 'recurrence_value1'),
            ('start_time', 'st1'),
            ('end_time', 'et1'),
            ('name', 'test_name')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_policy.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_policy.assert_called_with(
            alarm_id='alarm1',
            cool_down_time=1,
            name='test_name',
            scaling_group_id='group1',
            scaling_policy_action={'operation': 'ADD', 'instance_number': 7},
            scheduled_policy={
                'launch_time': 'launch_time1',
                'recurrence_type': 'recurrence_type1',
                'recurrence_value': 'recurrence_value1',
                'start_time': 'st1',
                'end_time': 'et1'
            },
            type='ALARM'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteAutoScalingPolicy(TestAutoScalingPolicy):

    def setUp(self):
        super(TestDeleteAutoScalingPolicy, self).setUp()

        self.cmd = policy.DeleteAutoScalingPolicy(self.app, None)

        self.client.delete_policy = mock.Mock()

    def test_delete(self):
        arglist = [
            'policy1',
            'policy2',
        ]
        verifylist = [
            ('policy', ['policy1', 'policy2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_policy.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call('policy1'), mock.call('policy2')]

        self.client.delete_policy.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_policy.call_count)


class TestUpdateAutoScalingPolicy(TestAutoScalingPolicy):

    columns = ['ID', 'Name', 'scaling_group_id', 'status',
               'type', 'alarm_id', 'scheduled_policy',
               'scaling_policy_action', 'cool_down_time'
               ]

    _obj = fakes.FakePolicy.create_one()

    data = (
        _obj.id,
        _obj.name,
        _obj.scaling_group_id,
        _obj.status,
        _obj.type,
        _obj.alarm_id,
        _obj.scheduled_policy,
        _obj.scaling_policy_action,
        _obj.cool_down_time,
    )

    def setUp(self):
        super(TestUpdateAutoScalingPolicy, self).setUp()

        self.cmd = policy.UpdateAutoScalingPolicy(self.app, None)

        self.client.update_policy = mock.Mock()

    def test_create(self):
        arglist = [
            '--group', 'group1',
            '--type', 'ALARM',
            '--cool-down-time', '1',
            '--alarm-id', 'alarm1',
            '--action-operation', 'ADD',
            '--action-instance-number', '7',
            '--launch-time', 'launch_time1',
            '--recurrence-type', 'recurrence_type1',
            '--recurrence-value', 'recurrence_value1',
            '--start-time', 'st1',
            '--end-time', 'et1',

            'policy1'
        ]
        verifylist = [
            ('group', 'group1'),
            ('cool_down_time', 1),
            ('type', 'ALARM'),
            ('alarm_id', 'alarm1'),
            ('action_operation', 'ADD'),
            ('action_instance_number', 7),
            ('launch_time', 'launch_time1'),
            ('recurrence_type', 'recurrence_type1'),
            ('recurrence_value', 'recurrence_value1'),
            ('start_time', 'st1'),
            ('end_time', 'et1'),
            ('policy', 'policy1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_policy.side_effect = [
            self._obj
        ]

        self.client.get_policy.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_policy.assert_called_with(
            self._obj,
            alarm_id='alarm1',
            cool_down_time=1,
            scaling_group_id='group1',
            scaling_policy_action={'operation': 'ADD', 'instance_number': 7},
            scheduled_policy={
                'launch_time': 'launch_time1',
                'recurrence_type': 'recurrence_type1',
                'recurrence_value': 'recurrence_value1',
                'start_time': 'st1',
                'end_time': 'et1'
            },
            type='ALARM'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestEnableAutoScalingPolicy(TestAutoScalingPolicy):

    def setUp(self):
        super(TestEnableAutoScalingPolicy, self).setUp()

        self.cmd = policy.EnableAutoScalingPolicy(self.app, None)

        self.client.resume_policy = mock.Mock()

    def test_enable(self):
        arglist = [
            'policy1'
        ]
        verifylist = [
            ('policy', 'policy1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.resume_policy.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.resume_policy.assert_called()


class TestDisableAutoScalingPolicy(TestAutoScalingPolicy):

    def setUp(self):
        super(TestDisableAutoScalingPolicy, self).setUp()

        self.cmd = policy.DisableAutoScalingPolicy(self.app, None)

        self.client.pause_policy = mock.Mock()

    def test_enable(self):
        arglist = [
            'policy1'
        ]
        verifylist = [
            ('policy', 'policy1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.pause_policy.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.pause_policy.assert_called()


class TestExecuteAutoScalingPolicy(TestAutoScalingPolicy):

    def setUp(self):
        super(TestExecuteAutoScalingPolicy, self).setUp()

        self.cmd = policy.ExecuteAutoScalingPolicy(self.app, None)

        self.client.execute_policy = mock.Mock()

    def test_enable(self):
        arglist = [
            'policy1'
        ]
        verifylist = [
            ('policy', 'policy1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.execute_policy.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.execute_policy.assert_called()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.ctsv3.v3 import tracker
EXAMPLE = {
    "tracker_type": "system",
    "tracker_name": "system",
    "obs_info": {
        "is_obs_created": False,
        "bucket_name": "test-data-tracker",
        "file_prefix_name": "11"
    },
}


class TestTracker(base.TestCase):
    def test_basic(self):
        sot = tracker.Tracker()
        self.assertEqual('/trackers', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = tracker.Tracker(**EXAMPLE)
        self.assertEqual(EXAMPLE['tracker_type'], sot.tracker_type)
        self.assertEqual(EXAMPLE['tracker_name'], sot.tracker_name)
        self.assertEqual(EXAMPLE['obs_info']['is_obs_created'],
                         sot.obs_info.is_obs_created)
        self.assertEqual(EXAMPLE['obs_info']['bucket_name'],
                         sot.obs_info.bucket_name)
        self.assertEqual(EXAMPLE['obs_info']['file_prefix_name'],
                         sot.obs_info.file_prefix_name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource
from openstack import utils


class Whitelist(resource.Resource):
    base_path = '/vpc-endpoint-services/%(endpoint_service_id)s/permissions'
    resources_key = 'permissions'

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'permission', 'sort_key', 'sort_dir', 'limit', 'offset'
    )

    # URI properties
    endpoint_service_id = resource.URI('endpoint_service_id')

    # Properties
    #: Specifies when the whitelist record is added.
    created_at = resource.Body('created_at')
    #: Specifies the description of a whitelist record of a VPC endpoint
    #:  service.
    description = resource.Body('description')
    #: Specifies the unique ID of the permission.
    id = resource.Body('id')
    #: Lists the whitelist records.
    permission = resource.Body('permission')

    def _action(self, session, action, domains=[]):
        """Preform actions on the request body."""
        uri = utils.urljoin(self.base_path % self._uri.attributes, 'action')
        for ix, domain in enumerate(domains):
            if not domain.startswith('iam:domain::'):
                domains[ix] = 'iam:domain::' + domain

        body = {'permissions': domains, 'action': action}
        response = session.post(uri, json=body)
        exceptions.raise_from_response(response)
        for raw_resource in response.json()[self.resources_key]:
            yield Whitelist.existing(permission=raw_resource)

    def add(self, session, domains=[]):
        """Add whitelist."""
        return self._action(session, 'add', domains)

    def remove(self, session, domains=[]):
        """Remove whitelist."""
        return self._action(session, 'remove', domains)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.cbr.v3 import restore as _restore


EXAMPLE = {
    'mappings': [{
        'backup_id': 'backup',
        'volume_id': 'volume',
    }],
    'power_on': True,
    'server_id': uuid.uuid4(),
    'volume_id': uuid.uuid4(),
    'resource_id': uuid.uuid4(),
}


class TestRestore(base.TestCase):

    def setUp(self):
        super(TestRestore, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _restore.Restore()
        self.sot_expected = _restore.Restore(**EXAMPLE)

    def test_basic(self):
        sot = _restore.Restore()
        self.assertEqual('restore', sot.resource_key)
        self.assertEqual('', sot.resources_key)
        self.assertEqual('/backups/%(backup_id)s/restore',
                         sot.base_path)
        self.assertFalse(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)
        self.assertDictEqual({
            'limit': 'limit',
            'marker': 'marker',
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        sot = _restore.Restore(**EXAMPLE)
        self.assertTrue(sot.power_on)
        self.assertEqual(
            EXAMPLE['server_id'],
            sot.server_id)
        self.assertEqual(
            EXAMPLE['volume_id'],
            sot.volume_id)
        self.assertEqual(
            EXAMPLE['resource_id'],
            sot.resource_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import openstack

from otcextensions.tests.functional.sdk.mrs import TestMrs

_logger = openstack._log.setup_logging('openstack')


class TestHost(TestMrs):

    def test_list(self):
        self.clusters = list(self.client.clusters())
        self.assertGreaterEqual(len(self.clusters), 0)
        if len(self.clusters) > 0:
            cluster = self.clusters[0]
            servers = self.client.hosts(cluster_id=cluster.id)
            for server in servers:
                _logger.debug(server)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.identity.v3 import agency_role


FAKE_ID = "945d-fe449be00148"
EXAMPLE = {
    "catalog": "BASE",
    "display_name": "Tenant Guest",
    "name": "readonly",
    "policy": {},
    "domain_id": None,
    "type": "AA",
    "id": "b32d99a7778d4fd9aa5bc616c3dc4e5f",
    "description": "Tenant Guest"
}


class TestAgencyRole(base.TestCase):

    def setUp(self):
        super(TestAgencyRole, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()

    def test_basic(self):
        sot = agency_role.AgencyRole()

        self.assertEqual(
            ('/v3.0/OS-AGENCY/%(role_ref_type)ss'
             '/%(role_ref_id)s/agencies/%(agency_id)s/roles'),
            sot.base_path)
        self.assertEqual('roles', sot.resources_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_head)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):

        sot = agency_role.AgencyRole(connection=self.cloud, **EXAMPLE)
        # Check how the override with "real" connection works
        self.assertEqual(
            ('https://identity.example.com/v3.0/OS-AGENCY/%(role_ref_type)ss'
             '/%(role_ref_id)s/agencies/%(agency_id)s/roles'),
            sot.base_path)

        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""DNAT v2 action implementations"""
import logging

from osc_lib import utils
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListDnatRules(command.Lister):

    _description = _("List DNAT Rules.")
    columns = (
        'Id',
        'Nat Gateway Id',
        'Port Id',
        'Private IP',
        'Floating Ip Address',
        'Protocol',
        'Status'
    )

    def get_parser(self, prog_name):
        parser = super(ListDnatRules, self).get_parser(prog_name)

        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_("Specifies the ID of the SNAT rule."),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_("Limit to fetch number of records."),
        )
        parser.add_argument(
            '--project-id',
            metavar='<project_id>',
            help=_("Specifies the project ID."),
        )
        parser.add_argument(
            '--nat-gateway-id',
            metavar='<nat_gateway_id>',
            help=_("Specifies the NAT gateway ID."),
        )
        parser.add_argument(
            '--port-id',
            metavar='<port_id>',
            help=_("Specifies the port ID of an ECS or a BMS."),
        )
        parser.add_argument(
            '--private-ip',
            metavar='<private_ip>',
            help=_("Specifies the private IP address, for example, "
                   "the IP address of a Direct Connect connection."),
        )
        parser.add_argument(
            '--internal-service-port',
            metavar='<internal_service_port>',
            help=_("Specifies port used by ECSs or BMSs to provide "
                   "services for external systems."),
        )
        parser.add_argument(
            '--floating-ip-id',
            metavar='<floating_ip_id>',
            help=_("Specifies the Floating IP ID."),
        )
        parser.add_argument(
            '--floating-ip-address',
            metavar='<floating_ip_address>',
            help=_("Specifies the Floating IP."),
        )
        parser.add_argument(
            '--external-service-port',
            metavar='<external_service_port>',
            help=_("Specifies the port for providing external services."),
        )
        parser.add_argument(
            '--protocol',
            metavar='<protocol>',
            help=_("Specifies the protocol type."
                   "Currently, TCP, UDP, and ANY are supported."),
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_("Specifies the status of the DNAT rule.\n"
                   "ACTIVE: The resource status is normal.\n"
                   "PENDING_CREATE: The resource is being created.\n"
                   "PENDING_UPDATE: The resource is being updated.\n"
                   "PENDING_DELETE: The resource is being deleted.\n"
                   "EIP_FREEZED: The EIP of the resource is frozen.\n"
                   "INACTIVE: The resource status is abnormal."),
        )
        parser.add_argument(
            '--admin-state-up',
            metavar='<admin_state_up>',
            help=_("Specifies whether the DNAT rule is enabled or "
                   "disabled. The value can be:\n"
                   "true: The DNAT rule is enabled.\n"
                   "false: The DNAT rule is disabled."),
        )
        parser.add_argument(
            '--created-at',
            metavar='<created_at>',
            help=_("Specifies when the DNAT rule is created (UTC time). "
                   "Its value rounds to 6 decimal places forseconds. "
                   "The format is yyyy-mm-ddhh:mm:ss."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat
        args_list = [
            'id',
            'limit',
            'project_id',
            'nat_gateway_id',
            'port_id',
            'private_ip',
            'internal_service_port',
            'floating_ip_id',
            'floating_ip_address',
            'external_service_port',
            'protocol',
            'status',
            'admin_state_up',
            'created_at'
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val
        data = client.dnat_rules(**attrs)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowDnatRule(command.ShowOne):
    _description = _("Show Dnat Rule details")

    def get_parser(self, prog_name):
        parser = super(ShowDnatRule, self).get_parser(prog_name)
        parser.add_argument(
            'dnat',
            metavar='<dnat_id>',
            help=_("Specifies the ID of the SNAT Rule"),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat
        obj = client.get_dnat_rule(parsed_args.dnat)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class CreateDnatRule(command.ShowOne):
    _description = _("Create new DNAT Rule")

    def get_parser(self, prog_name):
        parser = super(CreateDnatRule, self).get_parser(prog_name)
        parser.add_argument(
            '--nat-gateway-id',
            metavar='<nat_gateway_id>',
            required=True,
            help=_("Specifies the ID of the NAT Gateway."),
        )
        parser.add_argument(
            '--port-id',
            metavar='<port_id>',
            help=_("Specifies the port ID of an ECS or a BMS."),
        )
        parser.add_argument(
            '--private-ip',
            metavar='<private_ip>',
            help=_("Specifies the private IP address, for example, "
                   "the IP address of a Direct Connect connection."),
        )
        parser.add_argument(
            '--internal-service-port',
            metavar='<internal_service_port>',
            required=True,
            help=_("Specifies port used by ECSs or BMSs to provide "
                   "services for external systems."),
        )
        parser.add_argument(
            '--floating-ip-id',
            metavar="<floating_ip_id>",
            required=True,
            help=_("Specifies the Floating IP ID. Multiple "
                   "Floating IPs are separated using commas."),
        )
        parser.add_argument(
            '--external-service-port',
            metavar='<external_service_port>',
            required=True,
            help=_("Specifies the port for providing external services."),
        )
        parser.add_argument(
            '--protocol',
            metavar='<protocol>',
            required=True,
            help=_("Specifies the protocol type."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat

        args_list = [
            'nat_gateway_id',
            'port_id',
            'private_ip',
            'internal_service_port',
            'floating_ip_id',
            'external_service_port',
            'protocol'
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        obj = client.create_dnat_rule(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteDnatRule(command.Command):

    _description = _("Deletes Dnat Rule(s).")

    def get_parser(self, prog_name):
        parser = super(DeleteDnatRule, self).get_parser(prog_name)
        parser.add_argument(
            'dnat',
            metavar='<dnat_id>',
            nargs='+',
            help=_("Specifies the DNAT Rule(s) ID(s) to delete."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.nat
        result = 0
        for dnat in parsed_args.dnat:
            try:
                obj = client.get_dnat_rule(dnat)
                client.delete_dnat_rule(obj.id)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete DNAT rule with "
                          "ID '%(dnat)s': %(e)s"),
                          {'dnat': dnat, 'e': e})
        if result > 0:
            total = len(parsed_args.dnat)
            msg = (_("%(result)s of %(total)s DNAT Rule(s) failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.kms.v1 import key as _key

EXAMPLE = {
    'key_id': '0d0466b0-e727-4d9c-b35d-f84bb474a37f',
    'domain_id': '00074811d5c27c4f8d48bb91e4a1dcfd',
    'key_alias': 'caseuirpr',
    'realm': 'cn-north-1',
    'key_description': '123',
    'creation_date': '1502799822000',
    'scheduled_deletion_date': '',
    'key_state': '2',
    'default_key_flag': '0',
    'key_type': '1',
    'expiration_time': '1501578672000',
    'origin': 'kms'
}

EXAMPLE_LIST_P1 = {
    'keys': [
        '0d0466b0-e727-4d9c-b35d-f84bb474a37f',
    ],
    'key_details': [{
        'key_id': '0d0466b0-e727-4d9c-b35d-f84bb474a37f',
        'domain_id': '00074811d5c27c4f8d48bb91e4a1dcfd',
        'key_alias': 'caseuirpr',
        'realm': 'cn-north-1',
        'key_description': '123',
        'creation_date': '1502799822000',
        'scheduled_deletion_date': '',
        'key_state': '2',
        'default_key_flag': '0',
        'key_type': '1',
        'expiration_time': '1501578672000',
        'origin': 'kms'
    }],
    'next_marker': '1',
    'truncated': 'true'
}
EXAMPLE_LIST_P2 = {
    'keys': [
        '2e258389-bb1e-4568-a1d5-e1f50adf70ea',
    ],
    'key_details': [{
        'key_id': '2e258389-bb1e-4568-a1d5-e1f50adf70ea',
        'domain_id': '00074811d5c27c4f8d48bb91e4a1dcfd',
        'key_alias': 'casehvniz',
        'realm': 'cn-north-1',
        'key_description': '234',
        'creation_date': '1502799820000',
        'scheduled_deletion_date': '',
        'key_state': '2',
        'default_key_flag': '0',
        'key_type': '1',
        'expiration_time': '1501578673000',
        'origin': 'kms'
    }],
    'next_marker': '',
    'truncated': 'false'
}


class TestKey(base.TestCase):

    def setUp(self):
        super(TestKey, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()
        self.sot = _key.Key()

    def test_basic(self):
        sot = _key.Key()
        self.assertEqual('key_info', sot.resource_key)
        self.assertEqual('key_details', sot.resources_key)
        self.assertEqual('/kms', sot.base_path)
        self.assertEqual('/kms/list-keys', sot.list_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        sot = _key.Key.existing(**EXAMPLE)
        self.assertEqual(EXAMPLE['key_id'], sot.id)
        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)
        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)
        self.assertEqual(EXAMPLE['key_alias'], sot.key_alias)
        self.assertEqual(EXAMPLE['realm'], sot.realm)
        self.assertEqual(EXAMPLE['key_description'], sot.key_description)
        self.assertEqual(EXAMPLE['creation_date'], sot.creation_date)
        self.assertEqual(EXAMPLE['scheduled_deletion_date'],
                         sot.scheduled_deletion_date)
        self.assertEqual(EXAMPLE['key_state'], sot.key_state)
        self.assertEqual(EXAMPLE['default_key_flag'], sot.default_key_flag)
        self.assertEqual(EXAMPLE['key_type'], sot.key_type)
        # self.assertEqual(EXAMPLE['error_code'], sot.error_code)
        # self.assertEqual(EXAMPLE['error_msg'], sot.error_msg)

    def mocked_requests_list(*args, **kwargs):
        class MockResponse(object):
            def __init__(self, json_data, status_code):
                self.json_data = copy.deepcopy(json_data)
                self.status_code = status_code

            def json(self):
                return self.json_data

        marker = kwargs['json']['marker']
        if marker == '0':
            return MockResponse(EXAMPLE_LIST_P1, 200)
        elif marker == '1':
            return MockResponse(EXAMPLE_LIST_P2, 200)

        return MockResponse(None, 404)

    def test_list(self):
        self.sess.post.side_effect = self.mocked_requests_list

        result = list(
            self.sot.list(
                self.sess,
                limit=1,
                marker='0',
                key_state='state',
                sequence='seq'
            )
        )

        calls = [
            mock.call('/kms/list-keys', json={
                'limit': 1, 'marker': '1',
                'key_state': 'state', 'sequence': 'seq'}),
            mock.call('/kms/list-keys', json={
                'limit': 1, 'marker': '1',
                'key_state': 'state', 'sequence': 'seq'}),
        ]

        self.sess.post.assert_has_calls(calls)

        expected_list = [
            _key.Key.existing(**EXAMPLE_LIST_P1['key_details'][0]),
            _key.Key.existing(**EXAMPLE_LIST_P2['key_details'][0])
        ]

        self.assertEqual(expected_list, result)

    def test_get(self):
        sot = _key.Key(
            id=EXAMPLE['key_id'])
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'key_info': EXAMPLE.copy()}

        self.sess.post.return_value = mock_response

        result = sot.fetch(self.sess)

        self.sess.post.assert_called_once_with(
            '/kms/describe-key',
            json={
                'key_id': EXAMPLE['key_id']
            }
        )

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['key_id'], sot.id)
        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)
        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)
        self.assertEqual(EXAMPLE['key_alias'], sot.key_alias)
        self.assertEqual(EXAMPLE['realm'], sot.realm)
        self.assertEqual(EXAMPLE['key_description'], sot.key_description)
        self.assertEqual(EXAMPLE['creation_date'], sot.creation_date)
        self.assertEqual(EXAMPLE['scheduled_deletion_date'],
                         sot.scheduled_deletion_date)
        self.assertEqual(EXAMPLE['key_state'], sot.key_state)
        self.assertEqual(EXAMPLE['default_key_flag'], sot.default_key_flag)
        self.assertEqual(EXAMPLE['key_type'], sot.key_type)

    def test_create(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'key_info': {
                'key_id': EXAMPLE['key_id'],
                'domain_id': EXAMPLE['domain_id']
            }
        }

        self.sess.post.return_value = mock_response

        key = {
            'key_alias': EXAMPLE['key_alias'],
            'key_description': EXAMPLE['key_description'],
            'realm': EXAMPLE['realm'],
            'key_type': EXAMPLE['key_type'],
            # 'key_policy': 'policy',
            # 'key_usage': 'usage',
            # 'sequence': 'sequence'
        }

        sot = _key.Key.new(**key)
        result = sot.create(self.sess, prepend_key=False)

        call_args = self.sess.post.call_args_list[0]

        expected_json = copy.deepcopy(key)
        # expected_json.pop('scaling_group_id')
        #
        self.assertEqual('/kms/create-key', call_args[0][0])
        self.assertDictEqual(expected_json, call_args[1]['json'])

        self.sess.post.assert_called_once(
            # '/kms/create-key',
            # json=key
        )

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['key_id'], sot.id)
        self.assertEqual(EXAMPLE['domain_id'], sot.domain_id)
        self.assertEqual(EXAMPLE['key_alias'], sot.key_alias)
        self.assertEqual(EXAMPLE['realm'], sot.realm)
        self.assertEqual(EXAMPLE['key_description'], sot.key_description)

    def test_enable(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = _key.Key.existing(id=EXAMPLE['key_id'])

        sot.enable(self.sess)

        self.sess.post.assert_called_once_with(
            'kms/enable-key',
            json={'key_id': EXAMPLE['key_id']}
        )

    def test_disable(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = _key.Key.existing(id=EXAMPLE['key_id'])

        sot.disable(self.sess)

        self.sess.post.assert_called_once_with(
            'kms/disable-key',
            json={'key_id': EXAMPLE['key_id']}
        )

    def test_schedule_deletion(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = _key.Key.existing(id=EXAMPLE['key_id'])

        sot.schedule_deletion(self.sess, 10)

        self.sess.post.assert_called_once_with(
            'kms/schedule-key-deletion',
            json={'key_id': EXAMPLE['key_id'], 'pending_days': 10}
        )

    def test_cancel_deletion(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = _key.Key.existing(id=EXAMPLE['key_id'])

        sot.cancel_deletion(self.sess)

        self.sess.post.assert_called_once_with(
            'kms/cancel-key-deletion',
            json={'key_id': EXAMPLE['key_id']}
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class UrlDomainSpec(resource.Resource):
    id = resource.Body('id')
    domain = resource.Body('domain')
    cname_status = resource.Body('cname_status', type=int)
    ssl_id = resource.Body('ssl_id')
    ssl_name = resource.Body('ssl_name')
    min_ssl_version = resource.Body('min_ssl_version')
    verified_client_certificate_enabled = resource.Body(
        'verified_client_certificate_enabled',
        type=bool)
    is_has_trusted_root_ca = resource.Body('is_has_trusted_root_ca',
                                           type=bool)


class ApiGroup(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/api-groups'

    allow_list = True
    allow_create = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters('limit', 'offset', 'id',
                                              'name', 'precise_search')
    resources_key = 'groups'

    gateway_id = resource.URI('gateway_id')
    name = resource.Body('name')
    remark = resource.Body('remark')
    roma_app_id = resource.Body('roma_app_id')
    version = resource.Body('version')

    id = resource.Body('id')
    status = resource.Body('status')
    sl_domain = resource.Body('sl_domain')
    register_time = resource.Body('register_time')
    update_time = resource.Body('update_time')
    on_sell_status = resource.Body('on_sell_status', type=int)
    url_domains = resource.Body('url_domains',
                                type=list,
                                list_type=UrlDomainSpec)
    sl_domain_access_enabled = resource.Body('sl_domain_access_enabled',
                                             type=bool)
    sl_domains = resource.Body('sl_domains', type=list)
    call_limits = resource.Body('call_limits', type=int)
    time_interval = resource.Body('time_interval', type=int)
    time_unit = resource.Body('time_unit')
    is_default = resource.Body('is_default', type=bool)
    roma_app_name = resource.Body('roma_app_name')

    def _verify_name(self, session, gateway, **attrs):
        """Verify the name of an API group.
        """
        uri = f'/apigw/instances/{gateway.id}/api-groups/check'
        response = session.post(uri, json=attrs)
        exceptions.raise_from_response(response)
        return None

    def _update_group(self, session, gateway, **attrs):
        """Update API group.
        """
        url = f'/apigw/instances/{gateway.id}/api-groups/{self.id}'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class L7Rule(resource.Resource):
    resource_key = 'rule'
    resources_key = 'rules'
    base_path = '/elb/l7policies/%(l7policy_id)s/rules'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'admin_state_up', 'compare_type', 'id',
        'invert', 'key', 'provisioning_status',
        'type', 'rule_value', is_admin_state_up='admin_state_up'
    )

    # Properties
    #: Specifies the forwarding policy ID.
    l7policy_id = resource.URI('l7policy_id')

    #: Specifies how requests are matched and forwarded.
    compare_type = resource.Body('compare_type')
    #: Specifies the matching conditions of the forwarding rule.
    conditions = resource.Body('conditions', type=list)
    #: Specifies whether reverse matching is supported.
    invert = resource.Body('invert', type=bool)
    #: Specifies the administrative status of the forwarding rule.
    is_admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies the key of the match item.
    key = resource.Body('key')
    #: Specifies the project ID.
    project_id = resource.Body('project_id')
    #: Specifies the provisioning status of the forwarding rule.
    provisioning_status = resource.Body('provisioning_status')
    #: Specifies the match content.
    type = resource.Body('type')
    #: Specifies the value of the match item.
    rule_value = resource.Body('value')



# Licensed under the Apache License, Version 2.0 (the "License"); you may

# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource


class AsyncJobEntities(resource.Resource):
    image_id = resource.Body('image_id')
    image_name = resource.Body('image_name')
    process_percent = resource.Body('process_percent')


class AsyncJob(resource.Resource):
    allow_fetch = True
    requires_id = False

    project_id = resource.Body('project_id')
    job_id = resource.Body('job_id')
    status = resource.Body('status')
    job_type = resource.Body('job_type')
    begin_time = resource.Body('begin_time')
    end_time = resource.Body('end_time')
    error_code = resource.Body('error_code')
    fail_reason = resource.Body('fail_reason')
    entities = resource.Body('entities', type=AsyncJobEntities)

    def get(self, session, prepend_key=False, base_path=None):
        # Overriden here to override prepend_key default value
        return super(AsyncJob, self).get(session, prepend_key, base_path)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from otcextensions.sdk.swr.v2 import _base


class Organization(resource.Resource):
    base_path = '/manage/namespaces'
    resources_key = 'namespaces'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_fetch = True
    allow_list = True

    _query_mapping = resource.QueryParameters(namespace='namespace')

    #: Organization namespace.
    #: *Type:str*
    namespace = resource.Body('namespace')
    #: Organization ID
    #: *Type:str*
    id = resource.Body('id', type=str)
    #: Organization name
    #: *Type:str*
    name = resource.Body('name', type=str)
    #: Organization name
    #: *Type:str*
    creator_name = resource.Body('creator_name', type=str)
    #: User permission
    #: 7: Manage
    #: 3: Write
    #: 1: Read
    #: *Type:int*
    user_auth = resource.Body('auth', type=int)


class Auth(resource.Resource):
    #: Properties
    #: User ID
    user_id = resource.Body('user_id', type=str)
    #: Username
    user_name = resource.Body('user_name', type=str)
    #: User permission
    #: 7: Manage
    #: 3: Write
    #: 1: Read
    user_auth = resource.Body('auth', type=int)


class Permission(_base.Resource):
    base_path = '/manage/namespaces/%(namespace)s/access'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_commit = True
    allow_fetch = True
    allow_list = True

    commit_method = "PATCH"

    requires_id = False

    _query_mapping = resource.QueryParameters(namespace='namespace')

    #: Organization namespace.
    #: *Type:str*
    namespace = resource.URI('namespace')
    #: Information required for creating organization permissions.
    #: *Type:list*
    permissions = resource.Body('permissions', type=list, list_type=Auth)
    #: Permission ID
    #: *Type:int*
    id = resource.Body('id', type=int)
    #: Organization name
    #: *Type:str*
    name = resource.Body('name', type=str)
    #: Organization name
    #: *Type:str*
    creator_name = resource.Body('creator_name', type=str)
    #: Permissions of the current user
    #: *Type:dict*
    self_auth = resource.Body('self_auth', type=dict)
    #: Permissions of other users
    #: *Type:dict*
    others_auths = resource.Body('others_auths', type=list)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestVpcChannel(TestApiG):
    lb_channel = None
    gateway = "560de602c9f74969a05ff01d401a53ed"

    def setUp(self):
        super(TestVpcChannel, self).setUp()
        attrs = {
            "balance_strategy": 1,
            "member_type": "ip",
            "name": "VPC_demo",
            "port": 22,
        }
        vpc = self.client.create_vpc_channel(
            gateway=TestVpcChannel.gateway,
            **attrs)
        self.assertIsNotNone(vpc.id)
        TestVpcChannel.lb_channel = vpc

    def tearDown(self):
        super(TestVpcChannel, self).tearDown()
        self.client.delete_vpc_channel(
            gateway=TestVpcChannel.gateway,
            vpc_channel=TestVpcChannel.lb_channel.id)

    def test_01_get_vpc_channel(self):
        found = self.client.get_vpc_channel(
            gateway=TestVpcChannel.gateway,
            vpc_channel=TestVpcChannel.lb_channel['id']
        )
        self.assertIsNotNone(found.name)

    def test_02_vpc_channels(self):
        found = list(self.client.vpc_channels(gateway=TestVpcChannel.gateway))
        self.assertEqual(len(found), 1)

    def test_03_update_vpc_channel(self):
        attrs = {
            "balance_strategy": 1,
            "member_type": "ip",
            "name": "VPC_demo",
            "port": 222,
        }
        updated = self.client.update_vpc_channel(
            gateway=TestVpcChannel.gateway,
            vpc_channel=TestVpcChannel.lb_channel['id'],
            **attrs)
        self.assertEqual(updated.port, attrs["port"])

    def test_04_update_vpc_channel_healthcheck(self):
        attrs = {
            "http_code": "200",
            "path": "/vpc/demo",
            "port": 22,
            "method": "GET",
            "protocol": "http",
            "threshold_abnormal": 5,
            "threshold_normal": 2,
            "time_interval": 10,
            "timeout": 5,
            "enable_client_ssl": False
        }
        self.client.modify_vpc_channel_healthcheck(
            gateway=TestVpcChannel.gateway,
            vpc_channel=TestVpcChannel.lb_channel['id'],
            **attrs
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource

from otcextensions.common import exc


class Metadata(resource.Resource):

    # Properties
    #: Annotations
    annotations = resource.Body('annotations', type=dict)
    #: UUID
    #: *Type:str
    id = resource.Body('uid', alternate_id=True)
    #: Labels
    labels = resource.Body('labels', type=dict)
    #: Name
    #: *Type:str
    name = resource.Body('name')
    #: Create time
    #: *Type:str
    created_at = resource.Body('creationTimestamp')
    #: Update time
    #: *Type:str
    updated_at = resource.Body('updateTimestamp')


class StatusSpec(resource.Resource):
    # Properties
    #: Cluster status.
    status = resource.Body('phase')
    #: Job ID.
    job_id = resource.Body('jobID')


class Resource(resource.Resource):

    # Properties
    #: Kind
    kind = resource.Body('kind')
    #: api version
    api_version = resource.Body('apiVersion', default='v3')
    #: metadata
    metadata = resource.Body('metadata', type=Metadata)
    #: Cluster status
    status = resource.Body('status', type=StatusSpec)
    #: Creation date. A virtual attribute fetched from metadata
    created_at = resource.Body('created_at')
    #: Update date. A virtual attribute fetched from metadata
    updated_at = resource.Body('updated_at')

    def __getattribute__(self, name):
        """Return an attribute on this instance

        This is mostly a pass-through except for a specialization on
        the 'id' name, as this can exist under a different name via the
        `alternate_id` argument to resource.Body.
        """
        if name == 'id' or name == 'name':
            if name in self._body:
                return self._body[name]
            else:
                try:
                    metadata = self._body['metadata']
                    if name == 'id':
                        if isinstance(metadata, dict):
                            return metadata['uid']
                        elif isinstance(metadata, Metadata):
                            return metadata._body[metadata._alternate_id()]
                    else:
                        if isinstance(metadata, dict):
                            return metadata['name']
                        elif isinstance(metadata, Metadata):
                            return metadata.name
                except KeyError:
                    return None
        elif name == 'status.status':
            status = object.__getattribute__(self, 'status')
            return object.__getattribute__(status, 'status')
        elif name in ['job_id', 'status.job_id']:
            status = object.__getattribute__(self, 'status')
            return object.__getattribute__(status, 'job_id')
        elif name in ['created_at', 'updated_at']:
            metadata = object.__getattribute__(self, 'metadata')
            # CCE return completely broken time format:
            # 2021-08-26 11:46:31.841673 +0000 UTC
            # Since it is far away from being any existing standard - drop
            # TZ info away (it is anyway only UTC)
            # Project cleanup is the one who tries really to parse the date,
            # thus we need to ensure it is parsable
            timestamp = object.__getattribute__(metadata, name)
            return timestamp[0:24] if timestamp else None
        else:
            return object.__getattribute__(self, name)

    def _translate_response(
            self, response, has_body=None, error_message=None,
            resource_response_key=None):
        if has_body is None:
            has_body = self.has_body
        # NOTE: we only use our own exception parser
        exc.raise_from_response(response, error_message=error_message)
        if has_body:
            try:
                body = response.json()
                if self.resource_key and self.resource_key in body:
                    body = body[self.resource_key]

                body_attrs = self._consume_body_attrs(body)

                if self._store_unknown_attrs_as_properties:
                    body_attrs = self._pack_attrs_under_properties(
                        body_attrs, body)

                self._body.attributes.update(body_attrs)
                self._body.clean()
                if self.commit_jsonpatch or self.allow_patch:
                    # We need the original body to compare against
                    self._original_body = body_attrs.copy()
            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()
        dict.update(self, self.to_dict())

    def create(self, session, prepend_key=False, base_path=None):
        # Overriden here to override prepend_key default value
        return super(Resource, self).create(session, prepend_key, base_path)

    def delete(self, session, error_message=None):
        """Delete the remote resource based on this instance.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`

        :return: This :class:`Resource` instance.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_commit` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.ResourceNotFound` if
                 the resource was not found.
        """

        response = self._raw_delete(session)
        kwargs = {}
        if error_message:
            kwargs['error_message'] = error_message

        self._translate_response(response, has_body=True, **kwargs)
        return self

    @classmethod
    def list(cls, session, paginated=True, base_path=None, **params):
        """This method is a generator which yields resource objects.

        This resource object list generator handles pagination and takes query
        params for response filtering.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool paginated: ``True`` if a GET to this resource returns
                               a paginated series of responses, or ``False``
                               if a GET returns only one page of data.
                               **When paginated is False only one
                               page of data will be returned regardless
                               of the API's support of pagination.**
        :param str base_path: Base part of the URI for listing resources, if
                              different from
                              :data:`~openstack.resource.Resource.base_path`.
        :param dict params: These keyword arguments are passed through the
            :meth:`~openstack.resource.QueryParamter._transpose` method
            to find if any of them match expected query parameters to be
            sent in the *params* argument to
            :meth:`~keystoneauth1.adapter.Adapter.get`. They are additionally
            checked against the
            :data:`~openstack.resource.Resource.base_path` format string
            to see if any path fragments need to be filled in by the contents
            of this argument.

        :return: A generator of :class:`Resource` objects.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                 :data:`Resource.allow_list` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.InvalidResourceQuery` if query
                 contains invalid params.
        """
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        cls._query_mapping._validate(params, base_path=base_path)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        while uri:
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'},
                params=query_params.copy())
            exceptions.raise_from_response(response)

            if response.json() and 'items' in response.json():
                data = response.json()['items'] or []

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            for raw_resource in resources:

                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    **raw_resource)
                yield value

            return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter
from openstack.tests.unit import base

from otcextensions.sdk.cbr.v3 import vault

EXAMPLE = {
    'id': 'vault-id',
    'auto_bind': False,
    'auto_expand': False,
    'description': 'my vault',
    'billing': {
        'protect_type': 'backup',
        'object_type': 'server',
        'size': 40,
        'cloud_type': 'public',
        'consistent_level': 'crash_consistent',
        'charging_mode': 'post_paid'
    },
    'name': 'vault-name',
    'resources': [
        {
            'name': 'server-name',
            'type': 'OS::Nova::Server',
            'id': 'server-id',
        }
    ]
}


class TestVault(base.TestCase):

    def setUp(self):
        super(TestVault, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = None
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.retriable_status_codes = ()
        self.sot = vault.Vault()

    def test_basic(self):
        sot = vault.Vault()
        self.assertEqual('vault', sot.resource_key)
        self.assertEqual('vaults', sot.resources_key)
        self.assertEqual('/vaults', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = vault.Vault.existing(**EXAMPLE)
        self.assertFalse(sot.auto_bind)
        self.assertFalse(sot.auto_expand)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(
            EXAMPLE['billing']['protect_type'], sot.billing.protect_type)
        self.assertEqual(
            EXAMPLE['billing']['object_type'], sot.billing.object_type)
        self.assertEqual(
            EXAMPLE['billing']['size'], sot.billing.size)
        self.assertEqual(
            EXAMPLE['billing']['cloud_type'], sot.billing.cloud_type)
        self.assertEqual(
            EXAMPLE['billing']['consistent_level'],
            sot.billing.consistent_level)
        self.assertEqual(
            EXAMPLE['billing']['charging_mode'], sot.billing.charging_mode)
        self.assertEqual(
            EXAMPLE['name'], sot.name)
        self.assertEqual(
            EXAMPLE['resources'][0]['name'],
            sot.resources[0].name)
        self.assertEqual(
            EXAMPLE['resources'][0]['type'],
            sot.resources[0].type)
        self.assertEqual(
            EXAMPLE['resources'][0]['id'],
            sot.resources[0].id)

    def test_get(self):
        sot = vault.Vault.existing(
            id=EXAMPLE['id'])
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'vault': EXAMPLE.copy()}

        self.sess.get.return_value = mock_response

        result = sot.fetch(self.sess)

        self.sess.get.assert_called_once_with(
            'vaults/%s' %
            EXAMPLE['id'],
            microversion=None,
            params={},
            skip_cache=False
        )

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['id'], result.id)
        self.assertEqual(EXAMPLE['name'], result.name)

    def test_bind_policy(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = vault.Vault.existing(id=EXAMPLE['id'])

        sot.bind_policy(self.sess, 'policy_id')

        self.sess.post.assert_called_once_with(
            'vaults/%s/associatepolicy' % EXAMPLE['id'],
            json={'policy_id': 'policy_id'}
        )

    def test_unbind_policy(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = vault.Vault.existing(id=EXAMPLE['id'])

        sot.unbind_policy(self.sess, 'policy_id')

        self.sess.post.assert_called_once_with(
            'vaults/%s/dissociatepolicy' % EXAMPLE['id'],
            json={'policy_id': 'policy_id'}
        )

    def test_associate_resources(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        resources = [{
            'id': 'server1_id',
            'type': 'OS::Nova::Server'
        }, {
            'id': 'server2_id',
            'type': 'OS::Nova::Server'}
        ]
        sot = vault.Vault.existing(id=EXAMPLE['id'])

        sot.associate_resources(self.sess, resources)

        self.sess.post.assert_called_once_with(
            'vaults/%s/addresources' % EXAMPLE['id'],
            json={'resources': resources}
        )

    def test_dissociate_resources(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        resources = ['id1', 'id2']
        sot = vault.Vault.existing(id=EXAMPLE['id'])

        sot.dissociate_resources(self.sess, resources)

        self.sess.post.assert_called_once_with(
            'vaults/%s/removeresources' % EXAMPLE['id'],
            json={'resource_ids': resources}
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from osc_lib import exceptions

from otcextensions.osclient.kms.v1 import cmk
from otcextensions.tests.unit.osclient.kms.v1 import fakes


class TestCMK(fakes.TestKMS):

    def setUp(self):
        super(TestCMK, self).setUp()
        self.client = self.app.client_manager.kms


class TestListCMK(TestCMK):

    _objs = fakes.FakeCMK.create_multiple(3)

    columns = ('ID', 'key_alias', 'key_state')

    data = []

    for s in _objs:
        data.append((
            s.id,
            s.key_alias,
            s.key_state
        ))

    def setUp(self):
        super(TestListCMK, self).setUp()

        self.cmd = cmk.ListCMK(self.app, None)

        self.client.keys = mock.Mock()

    def test_list_default(self):
        arglist = [
            '--limit', '14',
            '--state', '2'
        ]

        verifylist = [
            ('limit', 14),
            ('state', 2)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.keys.side_effect = [
            self._objs
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.keys.assert_called_once_with(
            key_state=2,
            limit=14,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowCMK(TestCMK):

    _obj = fakes.FakeCMK.create_one()

    columns = ['ID', 'key_alias', 'domain_id', 'realm',
               'key_description', 'creation_date', 'scheduled_deletion_date',
               'key_state', 'key_type']

    data = (
        _obj.id,
        _obj.key_alias,
        _obj.domain_id,
        _obj.realm,
        _obj.key_description,
        _obj.creation_date,
        _obj.scheduled_deletion_date,
        _obj.key_state,
        _obj.key_type,
    )

    def setUp(self):
        super(TestShowCMK, self).setUp()

        self.cmd = cmk.ShowCMK(self.app, None)

        self.client.find_key = mock.Mock()

    def test_show(self):
        arglist = [
            'key1'
        ]

        verifylist = [
            ('key', 'key1'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_key.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_key.assert_called_once_with(
            'key1',
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateCMK(TestCMK):

    _obj = fakes.FakeCMK.create_one()

    columns = ['ID', 'key_alias', 'domain_id', 'realm',
               'key_description', 'creation_date', 'scheduled_deletion_date',
               'key_state', 'key_type']

    data = (
        _obj.id,
        _obj.key_alias,
        _obj.domain_id,
        _obj.realm,
        _obj.key_description,
        _obj.creation_date,
        _obj.scheduled_deletion_date,
        _obj.key_state,
        _obj.key_type,
    )

    def setUp(self):
        super(TestCreateCMK, self).setUp()

        self.cmd = cmk.CreateCMK(self.app, None)

        self.client.create_key = mock.Mock()

    def test_create(self):
        arglist = [
            'key1',
            '--description', 'dscr',
            '--realm', 'rlm',
            '--key_policy', 'pol',
            '--key_usage', 'usg',
            '--type', 'typ'
        ]

        verifylist = [
            ('alias', 'key1'),
            ('description', 'dscr'),
            ('realm', 'rlm'),
            ('key_policy', 'pol'),
            ('key_usage', 'usg'),
            ('type', 'typ')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_key.side_effect = [
            self._obj
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_key.assert_called_once_with(
            key_alias='key1',
            key_description='dscr',
            key_policy='pol',
            key_type='typ',
            key_usage='usg',
            realm='rlm'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestEnableCMK(TestCMK):

    _obj = fakes.FakeCMK.create_one()

    def setUp(self):
        super(TestEnableCMK, self).setUp()

        self.cmd = cmk.EnableCMK(self.app, None)

        self.client.enable_key = mock.Mock()
        self.client.find_key = mock.Mock()

    def test_enable(self):
        arglist = [
            'key1'
        ]

        verifylist = [
            ('key', 'key1'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_key.side_effect = [self._obj]
        self.client.enable_key.side_effect = [self._obj]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_key.assert_called_once_with('key1')
        self.client.enable_key.assert_called()


class TestDisableCMK(TestCMK):

    _obj = fakes.FakeCMK.create_one()

    def setUp(self):
        super(TestDisableCMK, self).setUp()

        self.cmd = cmk.DisableCMK(self.app, None)

        self.client.disable_key = mock.Mock()
        self.client.find_key = mock.Mock()

    def test_disable(self):
        arglist = [
            'key1'
        ]

        verifylist = [
            ('key', 'key1'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_key.side_effect = [self._obj]
        self.client.disable_key.side_effect = [self._obj]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_key.assert_called_once_with('key1')
        self.client.disable_key.assert_called_once()


class TestDeleteCMK(TestCMK):

    _obj = fakes.FakeCMK.create_one()

    def setUp(self):
        super(TestDeleteCMK, self).setUp()

        self.cmd = cmk.DeleteCMK(self.app, None)

        self.client.schedule_key_deletion = mock.Mock()
        self.client.find_key = mock.Mock()

    def test_delete(self):
        arglist = [
            'key1', '7'
        ]

        verifylist = [
            ('key', 'key1'),
            ('days', 7)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_key.side_effect = [self._obj]
        self.client.schedule_key_deletion.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_key.assert_called_once_with('key1')
        self.client.schedule_key_deletion.assert_called_once_with(
            key=self._obj,
            pending_days=7
        )

    def test_delete_out_of_range(self):
        arglist = [
            'key1', '3'
        ]

        verifylist = [
            ('key', 'key1'),
            ('days', 3)
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestCancelDeleteCMK(TestCMK):

    _obj = fakes.FakeCMK.create_one()

    def setUp(self):
        super(TestCancelDeleteCMK, self).setUp()

        self.cmd = cmk.CancelDeleteCMK(self.app, None)

        self.client.cancel_key_deletion = mock.Mock()
        self.client.find_key = mock.Mock()

    def test_delete(self):
        arglist = [
            'key1'
        ]

        verifylist = [
            ('key', 'key1'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_key.side_effect = [self._obj]
        self.client.cancel_key_deletion.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.find_key.assert_called_once_with('key1')
        self.client.cancel_key_deletion.assert_called_once_with(
            key=self._obj,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import resource


class AvailabilityZone(resource.Resource):
    base_path = '/elb/availability-zones'

    # capabilities
    allow_list = True

    # Properties
    #: Specifies the AZ code.
    code = resource.Body('code', type=str)
    #: Specifies the AZ status.
    state = resource.Body('state', type=str)

    @classmethod
    def list(cls, session, paginated=True, base_path=None,
             allow_unknown_params=False, **params):

        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path

        response = session.get(
            base_path,
            headers={"Accept": "application/json"},
            microversion=microversion)
        exceptions.raise_from_response(response)
        data = response.json()

        resources = data['availability_zones']

        if not isinstance(resources, list):
            resources = [resources]

        for raw_resource in resources[0]:
            value = cls.existing(
                microversion=microversion,
                connection=session._get_connection(),
                **raw_resource)
            yield value



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import utils


class IpAddressGroup(resource.Resource):
    resource_key = 'ipgroup'
    resources_key = 'ipgroups'
    base_path = '/elb/ipgroups'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'project_id', 'description', 'name', 'ip_list'
    )

    # Properties
    #: Provides the IP address group.
    id = resource.Body('id', type=str)
    #: Provides supplementary information about the IP address group.
    description = resource.Body('description', type=str)
    #: Specifies the project ID of the IP address group.
    project_id = resource.Body('project_id')
    #: Specifies the IP address group name.
    name = resource.Body('name', type=str)
    #: The ID of the project this load balancer is associated with.
    ip_list = resource.Body('ip_list', type=list, list_type=dict)
    #: Lists the IDs of listeners.
    listeners = resource.Body('listeners', type=list, list_type=dict)
    #: Specifies the time when the ip address group was created.
    created_at = resource.Body('created_at', type=str)
    #: Specifies the time when the ip address group was updated.
    updated_at = resource.Body('updated_at', type=str)

    def update_ip_addresses(self, session, **attrs):
        """Method to update ip addresses in ip address group

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param kwargs attrs: Dictionary to update ip address group
        """
        url = utils.urljoin(self.base_path, self.id,
                            '/iplist/create-or-update')
        body = {
            'ipgroup': attrs
        }
        response = session.post(url, json=body)
        return self._to_object(session, response)

    def delete_ip_addresses(self, session, **attrs):
        """Method to delete ip addresses from ip address group

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param kwargs attrs: Dictionary to delete ip address group
        """
        url = utils.urljoin(self.base_path, self.id, '/iplist/batch-delete')
        body = {
            'ipgroup': attrs
        }
        response = session.post(url, json=body)
        return self._to_object(session, response)

    def _to_object(self, session, response):
        has_body = (
            self.has_body
            if self.create_returns_body is None
            else self.create_returns_body
        )
        microversion = self._get_microversion(session)
        self.microversion = microversion
        self._translate_response(response, has_body=has_body)
        if self.has_body and self.create_returns_body is False:
            return self.fetch(session)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import copy
import mock

from openstack.tests.unit import base

from otcextensions.sdk.rds.v3 import instance

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    "id": IDENTIFIER,
    "availability_zone": "fake_az",
    "status": "ACTIVE",
    "name": "mysql-0820-022709-01",
    "port": 3306,
    "type": "Single",
    "region": "eu-de",
    "datastore": {
        "type": "mysql",
        "version": "5.7"
    },
    "replica_of_id": "some_fake",
    "created": "2018-08-20T02:33:49+0800",
    "updated": "2018-08-20T02:33:50+0800",
    "volume": {
        "type": "ULTRAHIGH",
        "size": 100
    },
    "ha": {
        "replication_mode": "sync"
    },
    "nodes": [{
        "id": "06f1c2ad57604ae89e153e4d27f4e4b8no01",
        "name": "mysql-0820-022709-01_node0",
        "role": "master",
        "status": "ACTIVE",
        "availability_zone": "eu-de-01"
    }],
    "private_ips": ["192.168.0.142"],
    "public_ips": ["10.154.219.187", "10.154.219.186"],
    "password": "fake_pass",
    "vpc_id": "b21630c1-e7d3-450d-907d-39ef5f445ae7",
    "subnet_id": "45557a98-9e17-4600-8aec-999150bc4eef",
    "security_group_id": "38815c5c-482b-450a-80b6-0a301f2afd97",
    "flavor_ref": "rds.mysql.s1.large",
    "switch_strategy": "",
    "backup_strategy": {
        "start_time": "19:00-20:00",
        "keep_days": 7
    },
    "maintenance_window": "02:00-06:00",
    "related_instance": [],
    "disk_encryption_id": "",
    "time_zone": ""
}


class TestInstance(base.TestCase):
    def setUp(self):
        super(TestInstance, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = None
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        # self.sess.get_project_id = mock.Mock(return_value=PROJECT_ID)
        self.sot = instance.Instance(**EXAMPLE)

    def test_basic(self):
        sot = instance.Instance()
        self.assertIsNone(sot.resource_key)
        self.assertEqual('instances', sot.resources_key)
        self.assertEqual('/instances', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = instance.Instance(**EXAMPLE)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['availability_zone'], sot.availability_zone)
        self.assertEqual(EXAMPLE['backup_strategy'], sot.backup_strategy)
        self.assertEqual(EXAMPLE['created'], sot.created_at)
        self.assertEqual(EXAMPLE['datastore'], sot.datastore)
        self.assertEqual(EXAMPLE['disk_encryption_id'], sot.disk_encryption_id)
        self.assertEqual(EXAMPLE['flavor_ref'], sot.flavor_ref)
        self.assertEqual(EXAMPLE['ha'], sot.ha)
        self.assertEqual(EXAMPLE['nodes'], sot.nodes)
        self.assertEqual(EXAMPLE['password'], sot.password)
        self.assertEqual(EXAMPLE['port'], sot.port)
        self.assertEqual(EXAMPLE['volume'], sot.volume)
        self.assertEqual(EXAMPLE['region'], sot.region)
        self.assertEqual(EXAMPLE['port'], sot.port)
        self.assertEqual(EXAMPLE['private_ips'], sot.private_ips)
        self.assertEqual(EXAMPLE['public_ips'], sot.public_ips)
        self.assertEqual(EXAMPLE['region'], sot.region)
        self.assertEqual(EXAMPLE['related_instance'], sot.related_instances)
        self.assertEqual(EXAMPLE['replica_of_id'], sot.replica_of_id)
        self.assertEqual(EXAMPLE['vpc_id'], sot.router_id)
        self.assertEqual(EXAMPLE['security_group_id'], sot.security_group_id)
        self.assertEqual(EXAMPLE['subnet_id'], sot.network_id)
        self.assertEqual(EXAMPLE['switch_strategy'], sot.switch_strategy)
        self.assertEqual(EXAMPLE['maintenance_window'], sot.maintenance_window)
        self.assertEqual(EXAMPLE['time_zone'], sot.time_zone)

        self.assertEqual({
            'datastore_type': 'datastore_type',
            'id': 'id',
            'limit': 'limit',
            'marker': 'marker',
            'name': 'name',
            'network_id': 'subnet_id',
            'offset': 'offset',
            'router_id': 'vpc_id',
            'type': 'type'
        }, sot._query_mapping._mapping)

    def test_translate_response(self):
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {
            'job_id': 'fake_job_id',
            'instance': copy.deepcopy(EXAMPLE)
        }
        response.headers = {}
        sot = instance.Instance.new()

        sot._translate_response(response)
        self.assertEqual('fake_job_id', sot.job_id)
        self.assertEqual(IDENTIFIER, sot.id)
        self.assertEqual(EXAMPLE['vpc_id'], sot.router_id)

    def test_fetch_restore_times(self):
        sot = instance.Instance(**EXAMPLE)
        restore_times = [{
            'start_time': 'some_start_time',
            'end_time': 'some_end_time'
        }]
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {
            'restore_time': restore_times}
        response.headers = {}
        self.sess.get.return_value = response

        rt = sot.fetch_restore_times(self.sess)

        self.assertEqual(restore_times, rt)
        self.assertEqual(restore_times, sot.restore_time)

    def test_fetch(self):
        sot = instance.Instance.existing(id=IDENTIFIER)

        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {'instances': [EXAMPLE]}
        response.headers = {}
        self.sess.get.return_value = response

        # Restore from backup
        rt = sot.fetch(self.sess)

        self.sess.get.assert_called_with(
            '/instances', headers={'Accept': 'application/json'},
            microversion=None, params={'id': IDENTIFIER})
        self.assertIsInstance(rt, instance.Instance)
        self.assertDictEqual(
            rt.to_dict(),
            instance.Instance(**EXAMPLE)
        )

    def test_restore(self):
        sot = instance.Instance(**EXAMPLE)
        job_id = 'fake_job'

        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {'job_id': job_id}
        response.headers = {}
        self.sess.post.return_value = response

        class FakeResource(object):
            def __init__(self, id):
                self.id = id

        # Restore from backup
        rt = sot.restore(self.sess, FakeResource('bid'))

        self.sess.post.assert_called_with(
            'instances/recovery',
            json={
                'source': {
                    'type': 'backup', 'backup_id': 'bid',
                    'instance_id': IDENTIFIER},
                'target': {'instance_id': IDENTIFIER}})

        # PIT
        rt = sot.restore(self.sess, None, 'rit')

        self.sess.post.assert_called_with(
            'instances/recovery',
            json={
                'source': {
                    'type': 'timestamp', 'restore_time': 'rit',
                    'instance_id': IDENTIFIER},
                'target': {'instance_id': IDENTIFIER}})

        self.assertEqual(job_id, rt)

    def test_get_backup_policy(self):
        sot = instance.Instance(**EXAMPLE)
        backup_policy = {
            'keep_days': 7,
            'start_time': '19:00-20:00',
            'period': '1,2'
        }
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {
            'backup_policy': backup_policy}
        response.headers = {}
        self.sess.get.return_value = response

        rt = sot.get_backup_policy(self.sess)

        self.sess.get.assert_called_with('instances/IDENTIFIER/backups/policy')

        self.assertEqual(backup_policy, rt)

    def test_set_backup_policy(self):
        sot = instance.Instance(**EXAMPLE)
        backup_policy = {
            'keep_days': 7,
            'start_time': '19:00-20:00',
            'period': '1,2'
        }

        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.put.return_value = response

        rt = sot.set_backup_policy(self.sess, **backup_policy)

        self.sess.put.assert_called_with(
            'instances/IDENTIFIER/backups/policy',
            json={'backup_policy': {'keep_days': 7, 'start_time':
                                    '19:00-20:00', 'period': '1,2'}})

        self.assertIsNone(rt)

    def test_tag_action(self):
        sot = instance.Instance(**EXAMPLE)
        tags = {
            'action': 'create',
            'tags': [{
                'key': 'test_tag', 'value': 'test_value'
            }]
        }

        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.post.return_value = response

        rt = sot._tag_action(self.sess, tags)

        self.sess.post.assert_called_with(
            'instances/IDENTIFIER/tags/action',
            json=tags)

        self.assertIsNone(rt)

    def test_add_tag(self):
        sot = instance.Instance(**EXAMPLE)
        test_tag_name = 'tagname'
        test_tag_value = 'tagvalue'
        sot._tag_action = mock.Mock()

        rt = sot.add_tag(self.sess, test_tag_name, test_tag_value)

        sot._tag_action.assert_called_with(
            self.sess, {'action': 'create', 'tags':
                        [{'key': test_tag_name, 'value': test_tag_value}]})

        self.assertIsNone(rt)

    def test_remove_tag(self):
        sot = instance.Instance(**EXAMPLE)
        test_tag_name = 'tagname'
        sot._tag_action = mock.Mock()

        rt = sot.remove_tag(self.sess, test_tag_name)

        sot._tag_action.assert_called_with(
            self.sess, {'action': 'delete', 'tags': [{'key': 'tagname'}]})

        self.assertIsNone(rt)

    def test_action(self):
        sot = instance.Instance(**EXAMPLE)
        action = {"restart": {}}
        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        self.sess.post.return_value = response

        rt = sot._action(self.sess, action)

        self.sess.post.assert_called_with(
            'instances/IDENTIFIER/action',
            json=action)

        self.assertIsNone(rt)

    def test_restart(self):
        sot = instance.Instance(**EXAMPLE)
        sot._action = mock.Mock()

        rt = sot.restart(self.sess)

        sot._action.assert_called_with(self.sess, {'restart': {}})

        self.assertIsNone(rt)

    def test_enlarge_volume(self):
        sot = instance.Instance(**EXAMPLE)
        size = 200
        sot._action = mock.Mock()

        rt = sot.enlarge_volume(self.sess, size)

        sot._action.assert_called_with(
            self.sess, {'enlarge_volume': {'size': size}})

        self.assertIsNone(rt)

    def test_update_flavor(self):
        sot = instance.Instance(**EXAMPLE)
        flavor_spec_code = 'dummy.spec.code'
        sot._action = mock.Mock()

        rt = sot.update_flavor(self.sess, flavor_spec_code)

        sot._action.assert_called_with(
            self.sess, {'resize_flavor': {'spec_code': flavor_spec_code}})

        self.assertIsNone(rt)
